import asyncio
import aiohttp
from os import environ, getenv
import ujson
from fastapi.responses import PlainTextResponse, StreamingResponse
from requests import post, Session, adapters
from loguru import logger
from SpectrumCommon.ServerUtils.ExceptionUtils import SpectrumException
from SpectrumCommon.Const.GB import GB_SW_UNKNOWN
import time

def getProcessTime(start):
    return time.strftime("%M:%S", time.gmtime(time.time()-start))

def log_error(url, data, response):
    logger.error(f'POST {url} status={response.status_code}')
    logger.error(f'POST {url} payload={data}')
    logger.error(f'POST {url} body={response.content}')
    try:
        json_response = ujson.loads(response.content)
        error_code = json_response.get('SWErrorCode', GB_SW_UNKNOWN)
        return json_response 
    except:
        error_code = GB_SW_UNKNOWN
        message = f'POST {url} status={response.status_code} payload={data} body={response.content}'
        raise SpectrumException(error_code,
                                status_code = response.status_code, 
                                message=f'POST {url} status={response.status_code} payload={data} body={response.content}')

# import io
def make_post(url, data=None, isJSON=True, headers={}, files = None, stream=False, backoff_factor=0.3):
    DEBUG_MODE = False

    if DEBUG_MODE == True:
        start = time.time()
        print(time.strftime("%I:%M %p") + ': _REQ ' + url)
    else:
        start = 0

    for key in ["AVENIR_SW_API_KEY"]:
        if key in environ.keys():
            headers[key] = getenv(key, "")

    session = Session()
    retries = adapters.Retry(total=3, backoff_factor=backoff_factor, status_forcelist=[ 500, 502, 503, 504 ])
    session.mount('http://', adapters.HTTPAdapter(max_retries=retries))
    session.mount('https://', adapters.HTTPAdapter(max_retries=retries))

    if isJSON:
        headers["Content-Type"] = "application/json"
        if stream:
            response = session.post(url, data=ujson.dumps(data), headers=headers, stream=stream)
            if response.status_code not in [200, 201]:
                log_error(url, data, response)
            # Pass the error straight back if POST failed for any reason
            if stream and response.ok:
                headers = {"Content-Disposition": response.headers["Content-Disposition"],
                        "Content-Type": response.headers["Content-Type"]}
                
                def iter_chunks(r):
                    for chunk in r.iter_content(1024 * 1024):
                        yield chunk

                return StreamingResponse(iter_chunks(response), headers=headers,  media_type='.zip')
        else:
            response = session.post(url, data=ujson.dumps(data), headers=headers)
            if response.status_code not in [200, 201]:
                log_error(url, data, response)
    else:
        response = session.post(url, data=data, files=files, headers=headers, stream=stream)
        if response.status_code not in [200, 201]:
            log_error(url, data, response)
        # Pass the error straight back if POST failed for any reason
        if stream and response.ok:
            headers = {"Content-Disposition": response.headers["Content-Disposition"],
                       "Content-Type": response.headers["Content-Type"]}
            
            def iter_chunks(r):
                for chunk in r.iter_content(1024 * 1024):
                    yield chunk

            return StreamingResponse(iter_chunks(response), headers=headers,  media_type='.zip')

    if DEBUG_MODE == True:
        print(time.strftime("%I:%M %p") + ': _RES ' + url + " time: " + getProcessTime(start))

    response = PlainTextResponse(response.text, status_code=response.status_code)
    return response

def make_get(url, isJSON=True, headers={}, files = None, stream=False):
    for key in ["AVENIR_SW_API_KEY"]:
        if key in environ.keys():
            headers[key] = getenv(key, "")

    session = Session()
    retries = adapters.Retry(total=3, backoff_factor=0.1, status_forcelist=[ 500, 502, 503, 504 ])
    session.mount('http://', adapters.HTTPAdapter(max_retries=retries))
    session.mount('https://', adapters.HTTPAdapter(max_retries=retries))

    if isJSON:
        headers["Content-Type"] = "application/json"
        if stream:
            response = session.get(url, headers=headers, stream=stream)
            # Pass the error straight back if GET failed for any reason
            if stream and response.ok:
                headers = {"Content-Disposition": response.headers["Content-Disposition"],
                        "Content-Type": response.headers["Content-Type"]}
                
                def iter_chunks(r):
                    for chunk in r.iter_content(1024 * 1024):
                        yield chunk

                return StreamingResponse(iter_chunks(response), headers=headers,  media_type='.zip')
        else:
            response = session.get(url, headers=headers)
            if response.status_code not in [200, 201]:
                logger.error(f'GET {url} status={response.status_code}')
    else:
        response = session.get(url, files=files, headers=headers, stream=stream)
        if response.status_code not in [200, 201]:
            logger.error(f'GET {url} status={response.status_code}')
        # Pass the error straight back if GET failed for any reason
        if stream and response.ok:
            headers = {"Content-Disposition": response.headers["Content-Disposition"],
                       "Content-Type": response.headers["Content-Type"]}
            
            def iter_chunks(r):
                for chunk in r.iter_content(1024 * 1024):
                    yield chunk

            return StreamingResponse(iter_chunks(response), headers=headers,  media_type='.zip')
    return PlainTextResponse(response.text, status_code=response.status_code)

#currently in development
def make_streaming_post(url, data=None, isJSON=True, headers={}, files = None):
    for key in ["AVENIR_SW_API_KEY"]:
        if key in environ.keys():
            headers[key] = getenv(key, "")

    if isJSON:
        headers["Content-Type"] = "application/json"
        response = post(url, data=ujson.dumps(data), headers=headers)
    else:
        # headers["Transfer-Encoding"] = 'chunked'
        response = post(url, data=data, files = files, headers=headers)#, stream=True)
    if response.status_code not in [200, 201]:
        logger.error(f'streaming POST {url} status={response.status_code}')
        logger.error(f'streaming POST {url} payload={data}')
        # Pass the error straight back if POST failed for any reason
    
    fileName = 'test.xlsx'
    # file_bytes = BytesIO() #response.content
    # for chunk in response.iter_content(chunk_size=10 * 1024):
    #     file_bytes.write(chunk)
    file_bytes = response.content
        
    if '.csv' in fileName:
        stream_response = StreamingResponse(
            iter_bytes(file_bytes),
            headers={"Content-Disposition": f"attachment; filename={fileName}",
                     "Content-Type":"text/csv; charset=utf-8"},
            media_type='.csv',
        )
    else:
        stream_response = StreamingResponse(
            file_bytes, #iter_bytes(file_bytes),
            media_type='application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
            headers={"Content-Disposition": "attachment; filename=test.xlsx", "Content-Length":str(len(file_bytes))}
        )  
    return stream_response#StreamingResponse(response.text, status_code=response.status_code)

def iter_bytes(file_bytes):
    yield file_bytes


async def async_make_post(url, data=None, isJSON=True, headers={}, files=None, stream=False, backoff_factor=0.3):
    for key in ["AVENIR_SW_API_KEY"]:
        if key in environ.keys():
            headers[key] = getenv(key, "")

    # aiohttp does not support built-in retries; you can implement manual retries if needed
    attempt = 0
    max_attempts = 3
    while attempt < max_attempts:
        try:
            async with aiohttp.ClientSession() as session:
                if isJSON:
                    headers["Content-Type"] = "application/json"
                    if stream:
                        async with session.post(url, data=ujson.dumps(data), headers=headers) as response:
                            if response.status not in [200, 201]:
                                logger.error(f'POST {url} status={response.status}')
                                logger.error(f'POST {url} payload={data}')
                                logger.error(f'POST {url} body={await response.text()}')
                                raise SpectrumException(f'POST {url} status={response.status} payload={data} body={await response.text()}')
                            # Stream response
                            return StreamingResponse(response.content.iter_chunked(1024 * 1024),
                                                    headers=dict(response.headers),
                                                    media_type='.zip')
                    else:
                        async with session.post(url, data=ujson.dumps(data), headers=headers) as response:
                            text = await response.text()
                            if response.status not in [200, 201]:
                                logger.error(f'POST {url} status={response.status}')
                                logger.error(f'POST {url} payload={data}')
                                logger.error(f'POST {url} body={text}')
                                raise SpectrumException(f'POST {url} status={response.status} payload={data} body={text}')
                            return PlainTextResponse(text, status_code=response.status)
                else:
                    # For files/multipart
                    async with session.post(url, data=data, headers=headers) as response:
                        text = await response.text()
                        if response.status not in [200, 201]:
                            logger.error(f'POST {url} status={response.status}')
                            logger.error(f'POST {url} payload={data}')
                            logger.error(f'POST {url} body={text}')
                            raise SpectrumException(f'POST {url} status={response.status} payload={data} body={text}')
                        if stream and response.status in [200, 201]:
                            return StreamingResponse(response.content.iter_chunked(1024 * 1024),
                                                    headers=dict(response.headers),
                                                    media_type='.zip')
                        return PlainTextResponse(text, status_code=response.status)
            break
        except Exception:
            attempt += 1
            if attempt >= max_attempts:
                raise
            await asyncio.sleep(backoff_factor * attempt)





from SpectrumCommon.Modvars.TB.TBNaomiSubnational import tb_apply_naomi_subnat
from SpectrumCommon.Modvars.DP.DPNaomiSubnational import dp_apply_naomi_subnat
from SpectrumCommon.Modvars.AM.AMNaomiSubnational import am_apply_naomi_subnat
from SpectrumCommon.Const.PJ.PJNTags import PJN_ModulesTag 
from SpectrumCommon.Const.GB.GBConst import GB_TB


def gb_apply_naomi_subnat(projection, subnat_region):
    
    projection = dp_apply_naomi_subnat(projection, subnat_region)       
    projection = am_apply_naomi_subnat(projection, subnat_region)
    # if projection['']
    if GB_TB in projection[PJN_ModulesTag]:
        projection = tb_apply_naomi_subnat(projection, subnat_region)

    return projection


from os import environ
import numpy as np

import SpectrumCommon.Const.GB as gbc
import SpectrumCommon.Modvars.GB.GBUtil as gbu

from AvenirCommon.Database import GB_get_db_json, GB_file_exists
import AvenirCommon.Util as acu

import SpectrumCommon.Const.RP.RPConst as rpc
import SpectrumCommon.Const.RP.RPConstLink as rpcl
import SpectrumCommon.Const.RP.RPDatabaseConst as rpdbc

import SpectrumCommon.Const.RP.RPModVarFields as rpmvf
import SpectrumCommon.Modvars.RP.RPModVarUtil as rpmvu

#######################################################################################################
#                                                                                                     #
#                                       Database loading methods                                      #
#                                                                                                     #
#######################################################################################################

def RP_load_country_defaults_DB(country_ID):
    country_defaults_DB_file_name = rpc.RP_COUNTRY_DEFAULTS_DB_DIR.replace('\\', '/') + '/' + country_ID + '_' + rpc.RP_COUNTRY_DEFAULTS_DB_CURR_VERSION + '.' + gbc.GB_JSON 
    
    if GB_file_exists(environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_RP_CONTAINER, country_defaults_DB_file_name):
        country_defaults_DB = GB_get_db_json(environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_RP_CONTAINER, country_defaults_DB_file_name) 
    
    else:
        country_defaults_DB = gbc.GB_NOT_FOUND

    return country_defaults_DB

def RP_load_country_defaults_sources_DB(country_ID):
    country_defaults_sources_DB_file_name = rpc.RP_COUNTRY_DEFAULTS_SOURCES_DB_DIR.replace('\\', '/') + '/' + country_ID + '_' + rpc.RP_COUNTRY_DEFAULTS_SOURCES_DB_CURR_VERSION + '.' + gbc.GB_JSON 

    if GB_file_exists(environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_RP_CONTAINER, country_defaults_sources_DB_file_name):
        country_defaults_sources_DB = GB_get_db_json(environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_RP_CONTAINER, country_defaults_sources_DB_file_name) 
    
    else:
        country_defaults_sources_DB = gbc.GB_NOT_FOUND

    return country_defaults_sources_DB

def RP_load_contraceptive_use_med_DB(country_ID):
    contraceptive_use_med_DB_file_name = rpc.RP_CONTRACEPTIVE_USE_MED_DB_DIR.replace('\\', '/') + '/' + country_ID + '_' + rpc.RP_CONTRACEPTIVE_USE_MED_DB_CURR_VERSION + '.' + gbc.GB_JSON 

    if GB_file_exists(environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_RP_CONTAINER, contraceptive_use_med_DB_file_name):
        contraceptive_use_med_DB = GB_get_db_json(environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_RP_CONTAINER, contraceptive_use_med_DB_file_name) 
    
    else:
        contraceptive_use_med_DB = gbc.GB_NOT_FOUND

    return contraceptive_use_med_DB

def RP_load_contraceptive_use_acc_DB(country_ID):
    contraceptive_use_acc_DB_file_name = rpc.RP_CONTRACEPTIVE_USE_ACC_DB_DIR.replace('\\', '/') + '/' + country_ID + '_' + rpc.RP_CONTRACEPTIVE_USE_ACC_DB_CURR_VERSION + '.' + gbc.GB_JSON 

    if GB_file_exists(environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_RP_CONTAINER, contraceptive_use_acc_DB_file_name):
        contraceptive_use_acc_DB = GB_get_db_json(environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_RP_CONTAINER, contraceptive_use_acc_DB_file_name) 
    
    else:
        contraceptive_use_acc_DB = gbc.GB_NOT_FOUND

    return contraceptive_use_acc_DB

def RP_load_urban_DB(country_ID):
    urban_DB_file_name = rpc.RP_URBAN_DB_DIR.replace('\\', '/') + '/' + country_ID + '_' + rpc.RP_URBAN_DB_CURR_VERSION + '.' + gbc.GB_JSON 

    if GB_file_exists(environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_RP_CONTAINER, urban_DB_file_name):
        urban_DB = GB_get_db_json(environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_RP_CONTAINER, urban_DB_file_name) 
    
    else:
        urban_DB = gbc.GB_NOT_FOUND

    return urban_DB

def RP_load_SSP1_DB(country_ID):
    SSP1_DB_file_name = rpc.RP_SSP1_TEMPS_DB_DIR.replace('\\', '/') + '/' + country_ID + '_' + rpc.RP_SSP1_TEMPS_DB_CURR_VERSION + '.' + gbc.GB_JSON 

    if GB_file_exists(environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_RP_CONTAINER, SSP1_DB_file_name):
        SSP1_DB = GB_get_db_json(environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_RP_CONTAINER, SSP1_DB_file_name) 
    
    else:
        SSP1_DB = gbc.GB_NOT_FOUND

    return SSP1_DB

def RP_load_SSP2_DB(country_ID):
    SSP2_DB_file_name = rpc.RP_SSP2_TEMPS_DB_DIR.replace('\\', '/') + '/' + country_ID + '_' + rpc.RP_SSP2_TEMPS_DB_CURR_VERSION + '.' + gbc.GB_JSON 

    if GB_file_exists(environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_RP_CONTAINER, SSP2_DB_file_name):
        SSP2_DB = GB_get_db_json(environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_RP_CONTAINER, SSP2_DB_file_name) 
    
    else:
        SSP2_DB = gbc.GB_NOT_FOUND

    return SSP2_DB

def RP_load_SSP3_DB(country_ID):
    SSP3_DB_file_name = rpc.RP_SSP3_TEMPS_DB_DIR.replace('\\', '/') + '/' + country_ID + '_' + rpc.RP_SSP3_TEMPS_DB_CURR_VERSION + '.' + gbc.GB_JSON 

    if GB_file_exists(environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_RP_CONTAINER, SSP3_DB_file_name):
        SSP3_DB = GB_get_db_json(environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_RP_CONTAINER, SSP3_DB_file_name) 
    
    else:
        SSP3_DB = gbc.GB_NOT_FOUND

    return SSP3_DB

def RP_load_SSP5_DB(country_ID):
    SSP5_DB_file_name = rpc.RP_SSP5_TEMPS_DB_DIR.replace('\\', '/') + '/' + country_ID + '_' + rpc.RP_SSP5_TEMPS_DB_CURR_VERSION + '.' + gbc.GB_JSON 

    if GB_file_exists(environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_RP_CONTAINER, SSP5_DB_file_name):
        SSP5_DB = GB_get_db_json(environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_RP_CONTAINER, SSP5_DB_file_name) 
    
    else:
        SSP5_DB = gbc.GB_NOT_FOUND

    return SSP5_DB

#######################################################################################################
#                                                                                                     #
#                                       Cost inputs                                                   #
#                                                                                                     #
#######################################################################################################

def RP_set_sector_defaults(country_defaults_DB, urban_DB, sectors_mv, first_year, final_year):
    first_year_idx = gbu.getYearIdx(first_year, first_year)
    final_year_idx = gbu.getYearIdx(final_year, first_year)

    education_sector_rec = rpmvu.get_RP_sector_rec(sectors_mv, rpc.RP_EDUCATION_SECTOR_MST_ID)
    health_sector_rec = rpmvu.get_RP_sector_rec(sectors_mv, rpc.RP_HEALTH_SECTOR_MST_ID)
    WASH_sector_rec = rpmvu.get_RP_sector_rec(sectors_mv, rpc.RP_WASH_SECTOR_MST_ID)
    food_security_sector_rec = rpmvu.get_RP_sector_rec(sectors_mv, rpc.RP_FOOD_SECURITY_SECTOR_MST_ID)
    urbanization_sector_rec = rpmvu.get_RP_sector_rec(sectors_mv, rpc.RP_URBANIZATION_SECTOR_MST_ID)
    energy_sector_rec = rpmvu.get_RP_sector_rec(sectors_mv, rpc.RP_ENERGY_SECTOR_MST_ID)
    economic_sector_rec = rpmvu.get_RP_sector_rec(sectors_mv, rpc.RP_ECONOMIC_SECTOR_MST_ID)

    # Education

    rpmvu.set_RP_edu_sector_prim_sch_start_age(education_sector_rec, country_defaults_DB[rpdbc.RP_AGE_START_PRIM_KEY_CDDB])
    rpmvu.set_RP_edu_sector_prim_sch_end_age(education_sector_rec, country_defaults_DB[rpdbc.RP_AGE_END_PRIM_KEY_CDDB])
    rpmvu.set_RP_edu_sector_sec_sch_start_age(education_sector_rec, country_defaults_DB[rpdbc.RP_AGE_START_SEC_KEY_CDDB])
    rpmvu.set_RP_edu_sector_sec_sch_end_age(education_sector_rec, country_defaults_DB[rpdbc.RP_AGE_END_SEC_KEY_CDDB])
    # rpmvu.set_RP_edu_sector_student_per_teacher_prim(education_sector_rec, country_defaults_DB[rpdbc.RP_TEACHER_RATIO_PRIM_KEY_CDDB])
    rpmvu.set_RP_edu_sector_avg_ann_teacher_sal_prim_GDP_ratio(education_sector_rec, country_defaults_DB[rpdbc.RP_PRIM_TEACHER_SALARY_KEY_CDDB])
    # rpmvu.set_RP_edu_sector_avg_ann_teacher_sal_sec(education_sector_rec, country_defaults_DB[rpdbc.RP_SEC_TEACHER_SALARY_KEY_CDDB])
    rpmvu.set_RP_edu_sector_prim_spend_per_student_perc_GDP(education_sector_rec, country_defaults_DB[rpdbc.RP_PRIM_COSTS_KEY_CDDB])
    rpmvu.set_RP_edu_sector_sec_spend_per_student_perc_GDP(education_sector_rec, country_defaults_DB[rpdbc.RP_SEC_COSTS_KEY_CDDB])

    goal_rec = rpmvu.get_RP_goal_rec(education_sector_rec, rpc.RP_EDU_1_INCR_PRIM_ENROLL_GOAL_MST_ID)

    for subgoal_ID in rpc.RP_EDU_1_SUBGOAL_MST_IDS:
        subgoal_rec = rpmvu.get_RP_subgoal_rec(goal_rec, subgoal_ID)

        if subgoal_rec != gbc.GB_NOT_FOUND:
            # Base year values are the same for all subgoals and different for each sex
            rpmvu.set_RP_subgoal_resulting_change_by_sex(subgoal_rec, gbc.GB_Female, first_year_idx, country_defaults_DB[rpdbc.RP_PRIM_ENROLL_FEMALES_KEY_CDDB])
            rpmvu.set_RP_subgoal_resulting_change_by_sex(subgoal_rec, gbc.GB_Male, first_year_idx, country_defaults_DB[rpdbc.RP_PRIM_ENROLL_MALES_KEY_CDDB])

            # Final year values vary by subgoal but are the same for each sex
            for sex in [gbc.GB_Male, gbc.GB_Female]:

                if subgoal_ID == rpc.RP_EDU_1_1_UNIVERSAL_PRIMARY_EDU_SUBG_MST_ID:
                    rpmvu.set_RP_subgoal_resulting_change_by_sex(subgoal_rec, sex, final_year_idx, 100)

                elif subgoal_ID == rpc.RP_EDU_1_2_ACHIEVE_GENDER_EQUALITY_SUBG_MST_ID:
                    value = max(country_defaults_DB[rpdbc.RP_PRIM_ENROLL_FEMALES_KEY_CDDB], country_defaults_DB[rpdbc.RP_PRIM_ENROLL_MALES_KEY_CDDB])
                    rpmvu.set_RP_subgoal_resulting_change_by_sex(subgoal_rec, sex, final_year_idx, value)
            
    goal_rec = rpmvu.get_RP_goal_rec(education_sector_rec, rpc.RP_EDU_2_INCR_SEC_ENROLL_GOAL_MST_ID)

    for subgoal_ID in rpc.RP_EDU_2_SUBGOAL_MST_IDS:
        subgoal_rec = rpmvu.get_RP_subgoal_rec(goal_rec, subgoal_ID)

        if subgoal_rec != gbc.GB_NOT_FOUND:
            # Base year values are the same for all subgoals and different for each sex
            rpmvu.set_RP_subgoal_resulting_change_by_sex(subgoal_rec, gbc.GB_Female, first_year_idx, country_defaults_DB[rpdbc.RP_SEC_ENROLL_FEMALES_KEY_CDDB])
            rpmvu.set_RP_subgoal_resulting_change_by_sex(subgoal_rec, gbc.GB_Male, first_year_idx, country_defaults_DB[rpdbc.RP_SEC_ENROLL_MALES_KEY_CDDB])

            # Final year values are the same for each subgoal and sex
            for sex in [gbc.GB_Male, gbc.GB_Female]:

                if subgoal_ID == rpc.RP_EDU_2_1_UNIVERSAL_PRIMARY_EDU_SUBG_MST_ID:
                    rpmvu.set_RP_subgoal_resulting_change_by_sex(subgoal_rec, sex, final_year_idx, 100)

                elif subgoal_ID == rpc.RP_EDU_2_2_ACHIEVE_GENDER_EQUALITY_SUBG_MST_ID:
                    rpmvu.set_RP_subgoal_resulting_change_by_sex(subgoal_rec, sex, final_year_idx, 
                        max(country_defaults_DB[rpdbc.RP_SEC_ENROLL_FEMALES_KEY_CDDB], country_defaults_DB[rpdbc.RP_SEC_ENROLL_MALES_KEY_CDDB])
                    )

    goal_rec = rpmvu.get_RP_goal_rec(education_sector_rec, rpc.RP_EDU_3_IMPR_STUDENT_TEACHER_RATIO_GOAL_MST_ID)
    
    subgoal_rec = rpmvu.get_RP_subgoal_rec(goal_rec, rpc.RP_EDU_3_1_REC_RATIO_SUBG_MST_ID)
    if subgoal_rec != gbc.GB_NOT_FOUND:
        rpmvu.set_RP_subgoal_resulting_change(subgoal_rec, first_year_idx, country_defaults_DB[rpdbc.RP_TEACHER_RATIO_PRIM_KEY_CDDB])
        rpmvu.set_RP_subgoal_resulting_change(subgoal_rec, final_year_idx, 25)

    subgoal_rec = rpmvu.get_RP_subgoal_rec(goal_rec, rpc.RP_EDU_3_2_USER_SPEC_RATIO_SUBG_MST_ID)
    if subgoal_rec != gbc.GB_NOT_FOUND:
        rpmvu.set_RP_subgoal_resulting_change(subgoal_rec, first_year_idx, country_defaults_DB[rpdbc.RP_TEACHER_RATIO_PRIM_KEY_CDDB])

    # Health

    rpmvu.set_RP_health_sector_avg_unit_cost_skilled_deliv(health_sector_rec, country_defaults_DB[rpdbc.RP_SKILLED_BIRTH_COSTS_KEY_CDDB])
    rpmvu.set_RP_health_sector_avg_unit_cost_ANC_4_plus_visits(health_sector_rec, country_defaults_DB[rpdbc.RP_ANC_COSTS_KEY_CDDB])
    rpmvu.set_RP_health_sector_perc_skilled_hlth_prof_doctors(health_sector_rec, country_defaults_DB[rpdbc.RP_PERC_DOCTORS_KEY_CDDB])
    rpmvu.set_RP_health_sector_perc_skilled_hlth_prof_nurses(health_sector_rec, country_defaults_DB[rpdbc.RP_PERC_NURSES_KEY_CDDB])
    rpmvu.set_RP_health_sector_avg_salary_doctors(health_sector_rec, country_defaults_DB[rpdbc.RP_DOCTOR_SALARY_PUBLIC_KEY_CDDB])
    rpmvu.set_RP_health_sector_avg_salary_nurses(health_sector_rec, country_defaults_DB[rpdbc.RP_NURSE_SALARY_PUBLIC_KEY_CDDB])

    goal_rec = rpmvu.get_RP_goal_rec(health_sector_rec, rpc.RP_HLTH_1_INCR_PERC_BIRTHS_SKILLED_GOAL_MST_ID)

    for subgoal_ID in rpc.RP_HLTH_1_SUBGOAL_MST_IDS:
        subgoal_rec = rpmvu.get_RP_subgoal_rec(goal_rec, subgoal_ID)

        if subgoal_rec != gbc.GB_NOT_FOUND:
            # Base year values are the same for all subgoals
            rpmvu.set_RP_subgoal_resulting_change(subgoal_rec, first_year_idx, country_defaults_DB[rpdbc.RP_SKILLED_BIRTHS_KEY_CDDB])
    
            if subgoal_ID == rpc.RP_HLTH_1_1_ALL_SUBG_MST_ID:
                rpmvu.set_RP_subgoal_resulting_change(subgoal_rec, final_year_idx, 100)

            elif subgoal_ID == rpc.RP_HLTH_1_2_X_PERC_BASED_Y_TARGETS_SUBG_MST_ID:
                rpmvu.set_RP_subgoal_resulting_change(subgoal_rec, final_year_idx, 90)

    goal_rec = rpmvu.get_RP_goal_rec(health_sector_rec, rpc.RP_HLTH_2_INCR_PERC_PREG_WOMEN_REC_VISITS_GOAL_MST_ID)

    for subgoal_ID in rpc.RP_HLTH_2_SUBGOAL_MST_IDS:
        subgoal_rec = rpmvu.get_RP_subgoal_rec(goal_rec, subgoal_ID)

        if subgoal_rec != gbc.GB_NOT_FOUND:
            # Base year values are the same for all subgoals
            rpmvu.set_RP_subgoal_resulting_change(subgoal_rec, first_year_idx, country_defaults_DB[rpdbc.RP_ANC_4_PLUS_KEY_CDDB])
    
            if subgoal_ID == rpc.RP_HLTH_2_1_ALL_SUBG_MST_ID:
                rpmvu.set_RP_subgoal_resulting_change(subgoal_rec, final_year_idx, 100)

            elif subgoal_ID == rpc.RP_HLTH_2_2_X_PERC_BASED_Y_TARGETS_SUBG_MST_ID:
                rpmvu.set_RP_subgoal_resulting_change(subgoal_rec, final_year_idx, 90)

    goal_rec = rpmvu.get_RP_goal_rec(health_sector_rec, rpc.RP_HLTH_3_INCR_ACCESS_HEALTH_PROF_GOAL_MST_ID)

    for subgoal_ID in rpc.RP_HLTH_3_SUBGOAL_MST_IDS:
        subgoal_rec = rpmvu.get_RP_subgoal_rec(goal_rec, subgoal_ID)

        if subgoal_rec != gbc.GB_NOT_FOUND:
            # Base year values are the same for all subgoals
            rpmvu.set_RP_subgoal_resulting_change(subgoal_rec, first_year_idx, country_defaults_DB[rpdbc.RP_PROFS_KEY_CDDB])
    
            if subgoal_ID == rpc.RP_HLTH_3_1_WHO_REC_SUBG_MST_ID:
                rpmvu.set_RP_subgoal_resulting_change(subgoal_rec, final_year_idx, 2.5)

    # WASH

    rpmvu.set_RP_WASH_sector_cap_costs_per_HH_impr_water(WASH_sector_rec, country_defaults_DB[rpdbc.RP_CAP_COST_PP_IMPROVED_WATER_KEY_CDDB])
    # Defunct?? rpmvu.set_RP_WASH_sector_ann_cost_menstrual_hlth_comm(WASH_sector_rec, country_defaults_DB[rpdbc.RP_UNIT_COST_1000_L_WATER_CDDB])
    rpmvu.set_RP_WASH_sector_cap_costs_per_HH_impr_sani(WASH_sector_rec, country_defaults_DB[rpdbc.RP_CAP_COST_PP_IMPROVED_SANI_KEY_CDDB])

    goal_rec = rpmvu.get_RP_goal_rec(WASH_sector_rec, rpc.RP_WASH_1_INCR_ACCESS_IMPR_WATER_SRC_GOAL_MST_ID)

    for subgoal_ID in rpc.RP_WASH_1_SUBGOAL_MST_IDS:
        subgoal_rec = rpmvu.get_RP_subgoal_rec(goal_rec, subgoal_ID)

        if subgoal_rec != gbc.GB_NOT_FOUND:
            # Base year values are the same for all subgoals
            rpmvu.set_RP_subgoal_resulting_change(subgoal_rec, first_year_idx, country_defaults_DB[rpdbc.RP_PERC_IMPROVED_WATER_SOURCE_KEY_CDDB])
    
            if subgoal_ID == rpc.RP_WASH_1_1_UNIVERSAL_ACCESS_SUBG_MST_ID:
                rpmvu.set_RP_subgoal_resulting_change(subgoal_rec, final_year_idx, 100)

    goal_rec = rpmvu.get_RP_goal_rec(WASH_sector_rec, rpc.RP_WASH_2_INCR_ACCESS_IMPR_SANI_GOAL_MST_ID)

    for subgoal_ID in rpc.RP_WASH_2_SUBGOAL_MST_IDS:
        subgoal_rec = rpmvu.get_RP_subgoal_rec(goal_rec, subgoal_ID)

        if subgoal_rec != gbc.GB_NOT_FOUND:
            # Base year values are the same for all subgoals
            rpmvu.set_RP_subgoal_resulting_change(subgoal_rec, first_year_idx, country_defaults_DB[rpdbc.RP_PERC_IMPROVED_SANI_KEY_CDDB])
    
            if subgoal_ID == rpc.RP_WASH_2_1_UNIVERSAL_ACCESS_SUBG_MST_ID:
                rpmvu.set_RP_subgoal_resulting_change(subgoal_rec, final_year_idx, 100)

    # Food security

    production = country_defaults_DB[rpdbc.RP_STAPLE_PRODUCTION_KEY_CDDB]
    area = country_defaults_DB[rpdbc.RP_AREA_HARVESTED_KEY_CDDB]
    rpmvu.set_RP_food_sector_prim_staple_crop(food_security_sector_rec, country_defaults_DB[rpdbc.RP_DOMINANT_CROP_KEY_CDDB])
    rpmvu.set_RP_food_sector_agricult_land(food_security_sector_rec, country_defaults_DB[rpdbc.RP_AGRICULTURAL_LAND_KEY_CDDB])
    rpmvu.set_RP_food_sector_crop_land(food_security_sector_rec, country_defaults_DB[rpdbc.RP_CROPLAND_HECTARES_KEY_CDDB])
    rpmvu.set_RP_food_sector_staple_crop_prod_baseline(food_security_sector_rec, country_defaults_DB[rpdbc.RP_STAPLE_PRODUCTION_KEY_CDDB])
    rpmvu.set_RP_food_sector_area_harv_staple_crop(food_security_sector_rec, country_defaults_DB[rpdbc.RP_AREA_HARVESTED_KEY_CDDB])
    rpmvu.set_RP_food_sector_tonnes_harv_perc_hect(food_security_sector_rec,  (production / area) if area > 0 else 0)
    rpmvu.set_RP_food_sector_ann_kg_per_year_consump_staple_crop(food_security_sector_rec, country_defaults_DB[rpdbc.RP_STAPLE_ANNUAL_CONSUMP_PER_CAP_KEY_CDDB])
    rpmvu.set_RP_food_sector_prod_costs_staple_crop(food_security_sector_rec, country_defaults_DB[rpdbc.RP_PROD_COST_PER_TON_DOMINANT_CROP_KEY_CDDB])
    rpmvu.set_RP_food_sector_daily_cost_food_aid(food_security_sector_rec, country_defaults_DB[rpdbc.RP_DAILY_COST_FOOD_AID_PER_PERSON_KEY_CDDB])

    # base_pop is bsaed on the Goal scenario, so we would have to calculate before setting the default?
    # ('Review Defaults'!F53*1000)/('Review Defaults'!F57*DemProj!B6)
    # (country_defaults_DB[rpdbc.RP_STAPLE_PRODUCTION_CDDB] * 1000) / (country_defaults_DB[rpdbc.RP_STAPLE_ANNUAL_CONSUMP_PER_CAP_CDDB] * base_pop)

    goal_rec = rpmvu.get_RP_goal_rec(food_security_sector_rec, rpc.RP_FOOD_1_INCR_PROP_DOM_PROD_STAPLE_CROP_GOAL_MST_ID)

    for subgoal_ID in rpc.RP_FOOD_1_SUBGOAL_MST_IDS:
        subgoal_rec = rpmvu.get_RP_subgoal_rec(goal_rec, subgoal_ID)

        if subgoal_rec != gbc.GB_NOT_FOUND:
            # Base year value - see commented out code above...
    
            if subgoal_ID == rpc.RP_FOOD_1_1_ALL_DOMEST_PROD_SUBG_MST_ID:
                rpmvu.set_RP_subgoal_resulting_change(subgoal_rec, final_year_idx, 100)

    goal_rec = rpmvu.get_RP_goal_rec(food_security_sector_rec, rpc.RP_FOOD_2_REDUCE_FOOD_INSEC_GOAL_MST_ID)

    for subgoal_ID in rpc.RP_FOOD_2_SUBGOAL_MST_IDS:
        subgoal_rec = rpmvu.get_RP_subgoal_rec(goal_rec, subgoal_ID)

        if subgoal_rec != gbc.GB_NOT_FOUND:
            # Base year values are the same for all subgoals
            rpmvu.set_RP_subgoal_resulting_change(subgoal_rec, first_year_idx, country_defaults_DB[rpdbc.RP_SEVERE_INSECURITY_KEY_CDDB])
    
            if subgoal_ID == rpc.RP_FOOD_2_2_HALVE_SEV_FOOD_INSECURITY_SUBG_MST_ID:
                rpmvu.set_RP_subgoal_resulting_change(subgoal_rec, final_year_idx, country_defaults_DB[rpdbc.RP_SEVERE_INSECURITY_KEY_CDDB] / 2)

    # Urbanization

    t1_UDB = gbu.getYearIdx(first_year, rpdbc.RP_FIRST_YEAR_CUDB)
    t2_UDB = gbu.getYearIdx(final_year, rpdbc.RP_FIRST_YEAR_CUDB)

    rpmvu.set_RP_urb_sector_perc_pop_urb_baseline_year(urbanization_sector_rec, urban_DB[rpdbc.RP_PERC_POP_LIVING_URBAN_AREAS_KEY_UDB][t1_UDB])
    rpmvu.set_RP_urb_sector_perc_pop_urb_endline_year(urbanization_sector_rec, urban_DB[rpdbc.RP_PERC_POP_LIVING_URBAN_AREAS_KEY_UDB][t2_UDB])
    rpmvu.set_RP_urb_sector_cost_per_HH_move_to_housing(urbanization_sector_rec, country_defaults_DB[rpdbc.RP_COST_BUILD_URBAN_HOUSEHOLD_KEY_CDDB])

    goal_rec = rpmvu.get_RP_goal_rec(urbanization_sector_rec, rpc.RP_URB_2_INCR_ACCESS_URBAN_HOUS_GOAL_MST_ID)

    for subgoal_ID in rpc.RP_URB_2_SUBGOAL_MST_IDS:
        subgoal_rec = rpmvu.get_RP_subgoal_rec(goal_rec, subgoal_ID)

        if subgoal_rec != gbc.GB_NOT_FOUND:
            # Base year values are the same for all subgoals
            rpmvu.set_RP_subgoal_resulting_change(subgoal_rec, first_year_idx, country_defaults_DB[rpdbc.RP_PERC_URBAN_INFORMAL_KEY_CDDB])
    
            if subgoal_ID == rpc.RP_URB_2_2_HALVE_POP_INFORMAL_SETTLEMENTS_SUBG_MST_ID:
                rpmvu.set_RP_subgoal_resulting_change(subgoal_rec, final_year_idx, country_defaults_DB[rpdbc.RP_PERC_URBAN_INFORMAL_KEY_CDDB] / 2)

    # Energy

    rpmvu.set_RP_energy_sector_energy_KWh_prod_per_cap_per_year(energy_sector_rec, country_defaults_DB[rpdbc.RP_ELECTRICITY_PROD_PER_CAP_KEY_CDDB])
    rpmvu.set_RP_energy_sector_cap_cost_per_HH_connect_grid(energy_sector_rec, country_defaults_DB[rpdbc.RP_LCOE_0_PER_PERSON_PER_DAY_KEY_CDDB])
    rpmvu.set_RP_energy_sector_cap_cost_per_HH_clean_cooking(energy_sector_rec, country_defaults_DB[rpdbc.RP_CLEAN_COOKING_CAP_COST_KEY_CDDB])
    rpmvu.set_RP_energy_sector_cost_per_KWh(energy_sector_rec, country_defaults_DB[rpdbc.RP_COST_PER_KWH_KEY_CDDB])

    goal_rec = rpmvu.get_RP_goal_rec(energy_sector_rec, rpc.RP_ENERGY_1_INCR_POP_ACC_ELEC_GOAL_MST_ID)

    for subgoal_ID in rpc.RP_ENERGY_1_SUBGOAL_MST_IDS:
        subgoal_rec = rpmvu.get_RP_subgoal_rec(goal_rec, subgoal_ID)

        if subgoal_rec != gbc.GB_NOT_FOUND:
            rpmvu.set_RP_subgoal_resulting_change(subgoal_rec, first_year_idx, country_defaults_DB[rpdbc.RP_PERC_POP_ELECTRICITY_KEY_CDDB])

            if subgoal_ID == rpc.RP_ENERGY_1_1_UNIV_ACCESS_SUBG_MST_ID:
                rpmvu.set_RP_subgoal_resulting_change(subgoal_rec, final_year_idx, 100)

    goal_rec = rpmvu.get_RP_goal_rec(energy_sector_rec, rpc.RP_ENERGY_2_INCR_PROD_ELEC_SUFF_LEVELS_GOAL_MST_ID)

    for subgoal_ID in rpc.RP_ENERGY_2_SUBGOAL_MST_IDS:
        subgoal_rec = rpmvu.get_RP_subgoal_rec(goal_rec, subgoal_ID)

        if subgoal_rec != gbc.GB_NOT_FOUND:
            rpmvu.set_RP_subgoal_resulting_change(subgoal_rec, first_year_idx, country_defaults_DB[rpdbc.RP_ELECTRICITY_PROD_PER_CAP_KEY_CDDB])

            if subgoal_ID == rpc.RP_ENERGY_2_1_1000_KWH_PER_PERSON_YEAR_MST_ID:
                rpmvu.set_RP_subgoal_resulting_change(subgoal_rec, final_year_idx, 1000)

    goal_rec = rpmvu.get_RP_goal_rec(energy_sector_rec, rpc.RP_ENERGY_3_INCR_HOUS_ACCESS_CLEAN_GOAL_MST_ID)

    for subgoal_ID in rpc.RP_ENERGY_3_SUBGOAL_MST_IDS:
        subgoal_rec = rpmvu.get_RP_subgoal_rec(goal_rec, subgoal_ID)

        if subgoal_rec != gbc.GB_NOT_FOUND:
            rpmvu.set_RP_subgoal_resulting_change(subgoal_rec, first_year_idx, country_defaults_DB[rpdbc.RP_CLEAN_COOKING_KEY_CDDB])

            if subgoal_ID == rpc.RP_ENERGY_3_1_UNIV_ACCESS_SUBG_MST_ID:
                rpmvu.set_RP_subgoal_resulting_change(subgoal_rec, final_year_idx, 100)

    # Economic

    rpmvu.set_RP_eco_sector_GDP_per_capita(economic_sector_rec, country_defaults_DB[rpdbc.RP_GDP_PER_CAP_KEY_CDDB])
    # base_pop is bsaed on the Goal scenario, so we would have to calculate before setting the default?
    # defunct rpmvu.set_RP_eco_sector_baseline_GDP(economic_sector_rec, (country_defaults_DB[rpdbc.RP_GDP_PER_CAP_CDDB] * base_pop) / 1000000)
    # defunct rpmvu.set_RP_eco_sector_GDP_growth_rate(economic_sector_rec, country_defaults_DB[rpdbc.RP_AVG_GROWTH_RATE_CDDB])
    rpmvu.set_RP_eco_sector_labor_partic_rate(economic_sector_rec, country_defaults_DB[rpdbc.RP_LABOR_BOTH_KEY_CDDB])
    rpmvu.set_RP_eco_sector_change_GDP_CDR(economic_sector_rec, country_defaults_DB[rpdbc.RP_GDP_BOOST_PERC_CHANGE_CDR_KEY_CDDB])
    
    goal_rec = rpmvu.get_RP_goal_rec(economic_sector_rec, rpc.RP_ECON_3_DECR_UNEMPLOYMENT_GOAL_MST_ID)

    for subgoal_ID in rpc.RP_ECON_3_SUBGOAL_MST_IDS:
        subgoal_rec = rpmvu.get_RP_subgoal_rec(goal_rec, subgoal_ID)

        if subgoal_rec != gbc.GB_NOT_FOUND:
            # Base year values are the same for all subgoals
            rpmvu.set_RP_subgoal_resulting_change(subgoal_rec, first_year_idx, country_defaults_DB[rpdbc.RP_UNEMPLOYMENT_KEY_CDDB])
    
            if subgoal_ID == rpc.RP_ECON_3_1_5_PERC_UNEMPLOYMENT_SUBG_MST_ID:
                rpmvu.set_RP_subgoal_resulting_change(subgoal_rec, final_year_idx, 5)
    
    goal_rec = rpmvu.get_RP_goal_rec(economic_sector_rec, rpc.RP_ECON_4_REDUCE_YOUTH_NEET_GOAL_MST_ID)

    for subgoal_ID in rpc.RP_ECON_4_SUBGOAL_MST_IDS:
        subgoal_rec = rpmvu.get_RP_subgoal_rec(goal_rec, subgoal_ID)

        if subgoal_rec != gbc.GB_NOT_FOUND:
            # Base year values are the same for all subgoals and different for each sex
            rpmvu.set_RP_subgoal_resulting_change_by_sex(subgoal_rec, gbc.GB_Female, first_year_idx, country_defaults_DB[rpdbc.RP_YOUTH_NEET_FEMALE_KEY_CDDB])
            rpmvu.set_RP_subgoal_resulting_change_by_sex(subgoal_rec, gbc.GB_Male, first_year_idx, country_defaults_DB[rpdbc.RP_YOUTH_NEET_MALE_KEY_CDDB])

            # Final year values are the same for each sex
            for sex in [gbc.GB_Male, gbc.GB_Female]:

                if subgoal_ID == rpc.RP_ECON_4_2_ACHIEVE_GENDER_EQUALITY_SUBG_MST_ID:
                    value = min(country_defaults_DB[rpdbc.RP_YOUTH_NEET_FEMALE_KEY_CDDB], country_defaults_DB[rpdbc.RP_YOUTH_NEET_MALE_KEY_CDDB])
                    rpmvu.set_RP_subgoal_resulting_change_by_sex(subgoal_rec, sex, final_year_idx, value)

def RP_set_climate_param_defaults(country_defaults_DB, climate_params):
    rpmvu.set_RP_climate_baseline_mean_med_temp(climate_params, country_defaults_DB[rpdbc.RP_BASELINE_AVG_TEMP_KEY_CDDB])
    rpmvu.set_RP_climate_perc_change_GDP_1_degree_warm(climate_params, country_defaults_DB[rpdbc.RP_1_DEGREE_GDP_KEY_CDDB])
    rpmvu.set_RP_climate_perc_change_staple_crop_1_degree_warm(climate_params, country_defaults_DB[rpdbc.RP_1_DEGREE_STAPLE_KEY_CDDB])
    rpmvu.set_RP_climate_perc_change_staple_crop_2_degree_warm(climate_params, country_defaults_DB[rpdbc.RP_2_DEGREE_STAPLE_KEY_CDDB])
    rpmvu.set_RP_climate_perc_change_staple_crop_3_degree_warm(climate_params, country_defaults_DB[rpdbc.RP_3_DEGREE_STAPLE_KEY_CDDB])
    rpmvu.set_RP_climate_perc_change_energy_needs_1_degree_warm(climate_params, country_defaults_DB[rpdbc.RP_1_DEGREE_ON_ENERGY_KEY_CDDB])

def RP_set_FP_scen_scale_up_defaults(contraceptive_use_med_DB, contraceptive_use_acc_DB, 
    FP_scen_scale_up_rec, scenario_goal_scale_up, scenario_counterfactual_scale_up, first_year, final_year):

    year_range = acu.GBRange(first_year, final_year)
    year_range_DB = acu.GBRange(rpdbc.RP_FIRST_YEAR_CUDB, rpdbc.RP_FINAL_YEAR_CUDB)

    # We're assuming here that the projection years fall within the database years (2020-2075). If they don't, this will break. Both databases have
    # the same set of years.
    t1_DB = gbu.getYearIdx(first_year, rpdbc.RP_FIRST_YEAR_CUDB)
    t2_DB = gbu.getYearIdx(final_year, rpdbc.RP_FIRST_YEAR_CUDB)
    
    # acc_t1 = contraceptive_use_acc_DB[rpdbc.RP_CONTRACEPTIVE_USE_KEY_CUDB][t1_DB]
    med_t1 = contraceptive_use_med_DB[rpdbc.RP_CONTRACEPTIVE_USE_KEY_CUDB][t1_DB]
    acc_t2 = contraceptive_use_acc_DB[rpdbc.RP_CONTRACEPTIVE_USE_KEY_CUDB][t2_DB]
    med_t2 = contraceptive_use_med_DB[rpdbc.RP_CONTRACEPTIVE_USE_KEY_CUDB][t2_DB]

    acc_avg_annual_change = (acc_t2 - med_t1) / (final_year - first_year) # Not a mistake, should be med and not acc
    med_avg_annual_change = (med_t2 - med_t1) / (final_year - first_year)

    for year in year_range:
        if year in year_range_DB:
            t = gbu.getYearIdx(year, first_year)

            # First year values for all three types should be median, first year from database.
            if year == first_year:
                FP_scen_scale_up_rec[rpmvf.RP_FP_SCEN_SC_ACCELERATED][t] = med_t1
                FP_scen_scale_up_rec[rpmvf.RP_FP_SCEN_SC_MEDIAN][t] = med_t1
                FP_scen_scale_up_rec[rpmvf.RP_FP_SCEN_SC_FLATLINE][t] = med_t1
                scenario_goal_scale_up[t] = med_t1
                scenario_counterfactual_scale_up[t] = med_t1

            else:
                FP_scen_scale_up_rec[rpmvf.RP_FP_SCEN_SC_ACCELERATED][t] = (
                    FP_scen_scale_up_rec[rpmvf.RP_FP_SCEN_SC_ACCELERATED][t - 1] + acc_avg_annual_change
                )

                FP_scen_scale_up_rec[rpmvf.RP_FP_SCEN_SC_MEDIAN][t] = (
                    FP_scen_scale_up_rec[rpmvf.RP_FP_SCEN_SC_MEDIAN][t - 1] + med_avg_annual_change 
                )

                FP_scen_scale_up_rec[rpmvf.RP_FP_SCEN_SC_FLATLINE][t] = FP_scen_scale_up_rec[rpmvf.RP_FP_SCEN_SC_FLATLINE][t - 1]

def RP_set_FP_method_cost_defaults(country_defaults_DB, FP_methods, avg_annual_unit_direct_costs):
    
    for m, value in enumerate(FP_methods):

        # If the method is active, set a default cost. 
        if value:

            if m == rpcl.RP_FP_M_CONDOM_M_CURR_ID:
                avg_annual_unit_direct_costs[m] = country_defaults_DB[rpdbc.RP_COST_CONDOMS_KEY_CDDB]

            elif m == rpcl.RP_FP_M_STERILIZ_F_CURR_ID:    
                avg_annual_unit_direct_costs[m] = country_defaults_DB[rpdbc.RP_COST_F_STERILIZATION_KEY_CDDB]

            elif m == rpcl.RP_FP_M_STERILIZ_M_CURR_ID:  
                avg_annual_unit_direct_costs[m] = country_defaults_DB[rpdbc.RP_COST_M_STERILIZATION_KEY_CDDB]

            elif m == rpcl.RP_FP_M_IUD_COPPER_10_YR_CURR_ID: 
                avg_annual_unit_direct_costs[m] = country_defaults_DB[rpdbc.RP_COST_IUD_KEY_CDDB]

            elif m == rpcl.RP_FP_M_INJECT_DEPO_3_MO_CURR_ID:
                avg_annual_unit_direct_costs[m] = country_defaults_DB[rpdbc.RP_COST_INJECTIONS_KEY_CDDB]

            elif m == rpcl.RP_FP_M_IMPLANT_JADELLE_5_YR_CURR_ID:
                avg_annual_unit_direct_costs[m] = country_defaults_DB[rpdbc.RP_COST_IMPLANT_KEY_CDDB]
            
            elif m == rpcl.RP_FP_M_PILL_STANDARD_CURR_ID:
                avg_annual_unit_direct_costs[m] = country_defaults_DB[rpdbc.RP_COST_PILL_KEY_CDDB]

            elif m == rpcl.RP_FP_M_VAG_BARRIER_CURR_ID:
                avg_annual_unit_direct_costs[m] = country_defaults_DB[rpdbc.RP_COST_VAG_BARR_KEY_CDDB]

            elif m == rpcl.RP_FP_M_OTHER_MODERN_METHODS_CURR_ID:
                avg_annual_unit_direct_costs[m] = country_defaults_DB[rpdbc.RP_COST_OTHER_MODERN_KEY_CDDB]

def RP_set_SSP_data(first_year, final_year, SSP1_DB, SSP2_DB, SSP3_DB, SSP5_DB):
    SSP_data = {}

    year_range = acu.GBRange(first_year, final_year)
    year_range_DB = acu.GBRange(rpdbc.RP_FIRST_YEAR_SSPDB, rpdbc.RP_FINAL_YEAR_SSPDB)

    num_years = final_year - first_year + 1

    if SSP1_DB != gbc.GB_NOT_FOUND:
        SSP_data[rpmvf.RP_SSP1] = np.full((num_years), 0.0).tolist()

        for year in year_range:
            if year in year_range_DB:
                t = gbu.getYearIdx(year, first_year)

                SSP_data[rpmvf.RP_SSP1][t] = SSP1_DB[rpdbc.RP_BASELINE_AVG_TEMP_KEY_SSPDB][t]

    if SSP2_DB != gbc.GB_NOT_FOUND:
        SSP_data[rpmvf.RP_SSP2] = np.full((num_years), 0.0).tolist()

        for year in year_range:
            if year in year_range_DB:
                t = gbu.getYearIdx(year, first_year)

                SSP_data[rpmvf.RP_SSP2][t] = SSP2_DB[rpdbc.RP_BASELINE_AVG_TEMP_KEY_SSPDB][t]

    if SSP3_DB != gbc.GB_NOT_FOUND:
        SSP_data[rpmvf.RP_SSP3] = np.full((num_years), 0.0).tolist()

        for year in year_range:
            if year in year_range_DB:
                t = gbu.getYearIdx(year, first_year)

                SSP_data[rpmvf.RP_SSP3][t] = SSP3_DB[rpdbc.RP_BASELINE_AVG_TEMP_KEY_SSPDB][t]

    if SSP5_DB != gbc.GB_NOT_FOUND:
        SSP_data[rpmvf.RP_SSP5] = np.full((num_years), 0.0).tolist()

        for year in year_range:
            if year in year_range_DB:
                t = gbu.getYearIdx(year, first_year)

                SSP_data[rpmvf.RP_SSP5][t] = SSP5_DB[rpdbc.RP_BASELINE_AVG_TEMP_KEY_SSPDB][t]

    return SSP_data

def RP_set_sources_defaults(country_default_sources_DB, sources_mv):

    sources_mv[rpmvf.RP_SRC_HH_SIZE] = country_default_sources_DB[rpdbc.RP_HH_SIZE_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_FP_METHOD_MIX] = 'FamPlan'
    sources_mv[rpmvf.RP_SRC_FP_EFFECTIVENESS] = 'FamPlan'
    sources_mv[rpmvf.RP_SRC_FP_UNIT_COSTS] = 'Adding it up - adjusted to 2024'
    sources_mv[rpmvf.RP_SRC_DP_FIRST_YEAR_POP] = 'WPP 2024'
    sources_mv[rpmvf.RP_SRC_FP_TFR] = 'FamPlan'

    sources_mv[rpmvf.RP_SRC_GOALS_PRIMARY_ENROLL_ALL] = country_default_sources_DB[rpdbc.RP_PERC_ENROLL_PRIM_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_GOALS_PRIMARY_ENROLL_MALE] = country_default_sources_DB[rpdbc.RP_PRIM_ENROLL_MALES_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_GOALS_PRIMARY_ENROLL_FEMALE] = country_default_sources_DB[rpdbc.RP_PRIM_ENROLL_FEMALES_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_GOALS_SECONDARY_ENROLL_ALL] = country_default_sources_DB[rpdbc.RP_PERC_ENROLL_SEC_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_GOALS_SECONDARY_ENROLL_MALE] = country_default_sources_DB[rpdbc.RP_SEC_ENROLL_MALES_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_GOALS_SECONDARY_ENROLL_FEMALE] = country_default_sources_DB[rpdbc.RP_SEC_ENROLL_FEMALES_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_GOALS_STUDENT_TEACHER_RATIO] = country_default_sources_DB[rpdbc.RP_TEACHER_RATIO_PRIM_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_GOALS_BIRTHS_ATTEND_SKILLED_PERS] = country_default_sources_DB[rpdbc.RP_SKILLED_BIRTH_COSTS_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_GOALS_PREG_W_4_PLUS_ANC_VISITS] = country_default_sources_DB[rpdbc.RP_ANC_4_PLUS_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_GOALS_HLTH_PROF_1000_POP] = country_default_sources_DB[rpdbc.RP_PROFS_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_GOALS_PERC_ACCESS_IMP_WATER] = country_default_sources_DB[rpdbc.RP_PERC_IMPROVED_WATER_SOURCE_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_GOALS_PERC_ACCESS_IMP_SANI] = country_default_sources_DB[rpdbc.RP_PERC_IMPROVED_SANI_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_GOALS_PERC_PROD_STAPLE_CROP] = country_default_sources_DB[rpdbc.RP_STAPLE_PRODUCTION_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_GOALS_PERC_SEV_FOOD_INSEC] = country_default_sources_DB[rpdbc.RP_SEVERE_INSECURITY_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_GOALS_PERC_URB_POP_INFORM_SET] = country_default_sources_DB[rpdbc.RP_PERC_URBAN_INFORMAL_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_GOALS_PERC_POP_ACCESS_ELEC] = country_default_sources_DB[rpdbc.RP_PERC_POP_ELECTRICITY_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_GOALS_KWH_PRODUCED] = country_default_sources_DB[rpdbc.RP_ELECTRICITY_PROD_PER_CAP_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_GOALS_PERC_ACCESS_CLEAN_COOK_TECH] = country_default_sources_DB[rpdbc.RP_CLEAN_COOKING_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_GOALS_UNEMPLOYMENT_RATE] = country_default_sources_DB[rpdbc.RP_UNEMPLOYMENT_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_GOALS_PERC_NEET_ALL] = country_default_sources_DB[rpdbc.RP_YOUTH_NEET_ALL_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_GOALS_PERC_NEET_MALES] = country_default_sources_DB[rpdbc.RP_YOUTH_NEET_MALE_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_GOALS_PERC_NEET_FEMALES] = country_default_sources_DB[rpdbc.RP_YOUTH_NEET_FEMALE_KEY_CDDB]

    sources_mv[rpmvf.RP_SRC_SECTOR_EDU_SCHOOL_AGE_RANGES] = country_default_sources_DB[rpdbc.RP_HH_SIZE_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_SECTOR_EDU_AVG_ANN_TEACHER_SAL_PRIM] = country_default_sources_DB[rpdbc.RP_PRIM_TEACHER_SALARY_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_SECTOR_EDU_ANN_COST_EDU_PRIM_STUDENT] = country_default_sources_DB[rpdbc.RP_PRIM_COSTS_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_SECTOR_EDU_ANN_COST_EDU_SEC_STUDENT] = country_default_sources_DB[rpdbc.RP_SEC_COSTS_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_SECTOR_HLTH_AVG_UNIT_COST_SKILLED_DELIV] = country_default_sources_DB[rpdbc.RP_SKILLED_BIRTH_COSTS_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_SECTOR_HLTH_AVG_UNIT_COST_ANC_4_PLUS_VISITS] = country_default_sources_DB[rpdbc.RP_ANC_COSTS_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_SECTOR_HLTH_PERC_SKILLED_HLTH_PROF_DOCTORS] = country_default_sources_DB[rpdbc.RP_PERC_DOCTORS_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_SECTOR_HLTH_PERC_SKILLED_HLTH_PROF_NURSES] = country_default_sources_DB[rpdbc.RP_PERC_NURSES_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_SECTOR_HLTH_AVG_SALARY_DOCTORS] = country_default_sources_DB[rpdbc.RP_DOCTOR_SALARY_PUBLIC_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_SECTOR_HLTH_AVG_SALARY_NURSES] = country_default_sources_DB[rpdbc.RP_NURSE_SALARY_PUBLIC_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_SECTOR_WASH_CAP_COSTS_PER_HH_IMPR_WATER] = country_default_sources_DB[rpdbc.RP_CAP_COST_PP_IMPROVED_WATER_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_SECTOR_WASH_CAP_COSTS_PER_HH_IMPR_SANI] = country_default_sources_DB[rpdbc.RP_CAP_COST_PP_IMPROVED_SANI_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_SECTOR_FOOD_PRIM_STAPLE_CROP] = country_default_sources_DB[rpdbc.RP_DOMINANT_CROP_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_SECTOR_FOOD_CROP_LAND] = country_default_sources_DB[rpdbc.RP_CROPLAND_HECTARES_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_SECTOR_FOOD_STAPLE_CROP_PROD_BASE] = country_default_sources_DB[rpdbc.RP_STAPLE_PRODUCTION_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_SECTOR_FOOD_TONNES_HARV_PERC_HECT] = country_default_sources_DB[rpdbc.RP_AREA_HARVESTED_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_SECTOR_FOOD_ANN_KG_PER_YEAR_CONSUMP_STAPLE_CROP] = country_default_sources_DB[rpdbc.RP_STAPLE_ANNUAL_CONSUMP_PER_CAP_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_SECTOR_FOOD_PROD_COSTS_STAPLE_CROP] = country_default_sources_DB[rpdbc.RP_PROD_COST_PER_TON_DOMINANT_CROP_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_SECTOR_FOOD_DAILY_COST_FOOD_AID] = country_default_sources_DB[rpdbc.RP_DAILY_COST_FOOD_AID_PER_PERSON_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_SECTOR_URB_PERC_POP_URB_BASE_END] = 'UN Population Division'
    sources_mv[rpmvf.RP_SRC_SECTOR_URB_COST_PER_HH_MOVE_TO_HOUSING] = country_default_sources_DB[rpdbc.RP_COST_BUILD_URBAN_HOUSEHOLD_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_SECTOR_ENERGY_CAP_COSTS_PER_HH_CONNECT_GRID] = country_default_sources_DB[rpdbc.RP_LCOE_0_PER_PERSON_PER_DAY_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_SECTOR_ENERGY_CAP_COSTS_PER_HH_CLEAN_COOKING] = country_default_sources_DB[rpdbc.RP_CLEAN_COOKING_CAP_COST_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_SECTOR_ECO_GDP_PER_CAPITA] = country_default_sources_DB[rpdbc.RP_GDP_PER_CAP_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_SECTOR_ECO_LABOR_PARTIC_RATE] = country_default_sources_DB[rpdbc.RP_LABOR_BOTH_KEY_CDDB]
    sources_mv[rpmvf.RP_SRC_SECTOR_ECO_CHANGE_GDP_CDR] = country_default_sources_DB[rpdbc.RP_GDP_BOOST_PERC_CHANGE_CDR_KEY_CDDB]

    sources_mv[rpmvf.RP_SRC_CLIM_BASELINE_MEAN_MED_TEMP] = 'SSP scenarios'
    sources_mv[rpmvf.RP_SRC_CLIM_PERC_CHANGE_GDP_1_DEG_WARM] = 'Roson & Sartori, 2016'
    sources_mv[rpmvf.RP_SRC_CLIM_PERC_CHANGE_ENERGY_NEEDS_1_DEG_WARM] = 'Roson & Sartori, 2016'
    sources_mv[rpmvf.RP_SRC_CLIM_PERC_CHANGE_STAPLE_CROP_1_DEG_WARM] = 'Roson & Sartori, 2016'
    sources_mv[rpmvf.RP_SRC_CLIM_PERC_CHANGE_STAPLE_CROP_2_DEG_WARM] = 'Roson & Sartori, 2016'
    sources_mv[rpmvf.RP_SRC_CLIM_PERC_CHANGE_STAPLE_CROP_3_DEG_WARM] = 'Roson & Sartori, 2016'


from .get_addresses import get_addresses 
import numpy as np
from scipy import sparse
import pickle

#clear all;
#for name in vars().keys():
#  del name

#gc.collect()

gps0 = {}
gps0['age']  = ['a1','a2','a3','a4','a5'] #gps0['age']  = {'a1','a2','a3','a4','a5'}
gps0['vacc'] = ['v0','v1','v2'] #gps0.vacc = {'v0','v1','v2'}; % After model calibration we can add these stages
gps0['hiv']  = ['h0','h1','hart'] #gps0.hiv  = {'h0','h1','hart'};
gps0['strains'] = ['ds','mdr'] #gps0.strains = {'ds','mdr'};
gps0['provs'] = ['pu','pr'] #gps0.provs = {'pu','pr'};

gps = {}
gps['age']  = ['a1','a2','a3','a4','a5'] #gps.age  = {'a1','a2','a3','a4','a5'};
gps['vacc'] = ['v0'] #gps.vacc = {'v0'};%,'v1','v2'}; % After model calibration we can add these stages
gps['hiv']  = ['h0','h1','hart'] #gps.hiv  = {'h0','h1','hart'};
gps['strains'] = ['ds','mdr'] #gps.strains = {'ds','mdr'};
gps['provs'] = ['pu','pr'] #gps.provs = {'pu','pr'};

gps1 = {}
gps1['age']  = ['a1','a2','a3','a4','a5'] #gps1.age  = {'a1','a2','a3','a4','a5'}; 
gps1['vacc'] = ['v0'] #gps1.vacc = {'v0'}; 
gps1['hiv']  = ['h0'] #gps1.hiv  = {'h0'}; 
gps1['strains'] = ['ds'] #gps1.strains = {'ds'}; 
gps1['provs'] = ['pr'] #gps1.provs = {'pr'};  

gps2 = {}
gps2['age']  = ['a1','a2','a3','a4','a5'] #gps2.age  = {'a1','a2','a3','a4','a5'}; 
gps2['vacc'] = ['v0']#gps2.vacc = {'v0'};  
gps2['hiv']  = ['h0'] #gps2.hiv  = {'h0'}; 
gps2['strains'] = ['ds','mdr'] #gps2.strains = {'ds','mdr'};
gps2['provs'] = ['pr'] #gps2.provs = {'pr'}; 

states0 = ['U'] #states0 = {'U'};                              % structured by hiv and vaccine status 
states1 = ['Lf','Ls'] #states1 = {'Lf','Ls'};                        % structured by hiv, vaccine and strain 
states2 = ['Isc','I','E','Rlo','Rhi','R'] #states2 = {'Isc','I','E','Rlo','Rhi','R'};    % structured by hiv and strain 
states3 = ['Dx','Tx','Tx2'] #states3 = {'Dx','Tx','Tx2'};                  % structured by hiv, strain and provider type

#% For the full model with vaccination
[i0, s0, d0, lim0] = get_addresses([states0, gps0['age'], gps0['hiv'], gps0['vacc']], {}, {}, {}, 0) #[i0, s0, d0, lim0] = get_addresses({states0, gps0.age, gps0.hiv, gps0.vacc}, [], [], [], 0);
[i0, s0, d0, lim0] = get_addresses([states1, gps0['age'], gps0['hiv'], gps0['vacc'], gps['strains']], i0, s0, d0, lim0)#[i0, s0, d0, lim0] = get_addresses({states1, gps0.age, gps0.hiv, gps0.vacc, gps.strains}, i0, s0, d0, lim0)
[i0, s0, d0, lim0] = get_addresses([states2, gps0['age'], gps0['hiv'], gps0['strains']], i0, s0, d0, lim0)#[i0, s0, d0, lim0] = get_addresses({states2, gps0.age, gps0.hiv, gps0.strains}, i0, s0, d0, lim0)
[i0, s0, d0, lim0] = get_addresses([states3, gps0['age'], gps0['hiv'], gps0['strains'], gps['provs']], i0, s0, d0, lim0)#[i0, s0, d0, lim0] = get_addresses({states3, gps0.age, gps0.hiv, gps0.strains, gps.provs}, i0, s0, d0, lim0)
d0 = [k for r, k in d0.items()] #d0 = chr(d0) #d0 = char(d0);

#% For the full model without vaccination
[i, s, d, lim] = get_addresses([states0, gps['age'], gps['hiv'], gps['vacc']], {}, {}, {}, 0)#[i, s, d, lim] = get_addresses({states0, gps.age, gps.hiv, gps.vacc}, [], [], [], 0);
[i, s, d, lim] = get_addresses([states1, gps['age'], gps['hiv'], gps['vacc'], gps['strains']], i, s, d, lim) #[i, s, d, lim] = get_addresses({states1, gps.age, gps.hiv, gps.vacc, gps.strains}, i, s, d, lim);
[i, s, d, lim] = get_addresses([states2, gps['age'], gps['hiv'], gps['strains']], i, s, d, lim) #[i, s, d, lim] = get_addresses({states2, gps.age, gps.hiv, gps.strains}, i, s, d, lim);
[i, s, d, lim] = get_addresses([states3, gps['age'], gps['hiv'], gps['strains'], gps['provs']], i, s, d, lim) #[i, s, d, lim] = get_addresses({states3, gps.age, gps.hiv, gps.strains, gps.provs}, i, s, d, lim);
d = [k for r, k in d.items()] #d = chr(d) #d = char(d);

#% For equilibrium model without mdr/hiv/vaccine/public sector 
[i1, s1, d1, lim1] = get_addresses([states0, gps1['age'], gps1['hiv'], gps1['vacc']], {}, {}, {}, 0) #[i1, s1, d1, lim1] = get_addresses({states0, gps1.age, gps1.hiv, gps1.vacc}, [], [], [], 0);
[i1, s1, d1, lim1] = get_addresses([states1, gps1['age'], gps1['hiv'], gps1['vacc'], gps1['strains']], i1, s1, d1, lim1)#[i1, s1, d1, lim1] = get_addresses({states1, gps1.age, gps1.hiv, gps1.vacc, gps1.strains}, i1, s1, d1, lim1);
[i1, s1, d1, lim1] = get_addresses([states2, gps1['age'], gps1['hiv'], gps1['strains']], i1, s1, d1, lim1)#[i1, s1, d1, lim1] = get_addresses({states2, gps1.age, gps1.hiv, gps1.strains}, i1, s1, d1, lim1);
[i1, s1, d1, lim1] = get_addresses([states3, gps1['age'], gps1['hiv'], gps1['strains'], gps1['provs']], i1, s1, d1, lim1)#[i1, s1, d1, lim1] = get_addresses({states3, gps1.age, gps1.hiv, gps1.strains, gps1.provs}, i1, s1, d1, lim1);
d1 = [k for r, k in d1.items()] #d1 = chr(d1)#d1 = char(d1);

#% Indtroducing mdr with the previous small model
[i2, s2, d2, lim2] = get_addresses([states0, gps2['age'], gps2['hiv'], gps2['vacc']], {}, {}, {}, 0) #[i2, s2, d2, lim2] = get_addresses({states0, gps2.age, gps2.hiv, gps2.vacc}, [], [], [], 0);
[i2, s2, d2, lim2] = get_addresses([states1, gps2['age'], gps2['hiv'], gps2['vacc'], gps2['strains']], i2, s2, d2, lim2)#[i2, s2, d2, lim2] = get_addresses({states1, gps2.age, gps2.hiv, gps2.vacc, gps2.strains}, i2, s2, d2, lim2);
[i2, s2, d2, lim2] = get_addresses([states2, gps2['age'], gps2['hiv'], gps2['strains']], i2, s2, d2, lim2)#[i2, s2, d2, lim2] = get_addresses({states2, gps2.age, gps2.hiv, gps2.strains}, i2, s2, d2, lim2);
[i2, s2, d2, lim2] = get_addresses([states3, gps2['age'], gps2['hiv'], gps2['strains'], gps2['provs']], i2, s2, d2, lim2) #[i2, s2, d2, lim2] = get_addresses({states3, gps2.age, gps2.hiv, gps2.strains, gps2.provs}, i2, s2, d2, lim2);
d2 = [k for r, k in d2.items()] #d2 = chr(d2) #d2 = char(d2);

s['infectious']      = np.concatenate([s['Isc'], s['I'], s['E'], s['Dx'], np.intersect1d(s['Tx'], s['mdr'])]) #s.infectious      = [s.Isc, s.I, s.E, s.Dx, intersect(s.Tx, s.mdr)];
s['infectious_wosc'] = np.concatenate([s['I'], s['E'], s['Dx'], np.intersect1d(s['Tx'], s['mdr'])]) #s.infectious_wosc = [s.I, s.E, s.Dx, intersect(s.Tx, s.mdr)];    % Infectious compartment without subclinical (contributing on TB death)
s['prevalent']       = np.unique(np.concatenate([s['infectious'], np.concatenate([s['Tx'],s['Tx2']])])) #s.prevalent       = unique([s.infectious, [s.Tx,s.Tx2]]);

s0['infectious']      = np.concatenate([s0['Isc'], s0['I'], s0['E'], s0['Dx'], np.intersect1d(s0['Tx'], s0['mdr'])]) #s0.infectious      = [s0.Isc, s0.I, s0.E, s0.Dx, intersect(s0.Tx, s0.mdr)];
s0['infectious_wosc'] = np.concatenate([s0['I'], s0['E'], s0['Dx'], np.intersect1d(s0['Tx'], s0['mdr'])]) #s0.infectious_wosc = [s0.I, s0.E, s0.Dx, intersect(s0.Tx, s0.mdr)];    % Infectious compartment without subclinical (contributing on TB death)
s0['prevalent']       = np.unique(np.concatenate([s0['infectious'], np.concatenate([s0['Tx'],s0['Tx2']])])) #s0.prevalent       = unique([s0.infectious, [s0.Tx,s0.Tx2]]);

s1['infectious']      = np.concatenate([s1['Isc'], s1['I'], s1['E'], s1['Dx']]) #s1.infectious      = [s1.Isc, s1.I, s1.E, s1.Dx];
s1['infectious_wosc'] = np.concatenate([s1['I'], s1['E'], s1['Dx']]) #s1.infectious_wosc = [s1.I, s1.E, s1.Dx];    % Infectious compartment without subclinical (contributing on TB death)
s1['prevalent']       = np.unique(np.concatenate([s1['infectious'], np.concatenate([s1['Tx'],s1['Tx2']])])) #s1.prevalent       = unique([s1.infectious, [s1.Tx,s1.Tx2]]);

s2['infectious']      = np.concatenate([s2['Isc'], s2['I'], s2['E'], s2['Dx'], np.intersect1d(s2['Tx'], s2['mdr'])]) #s2.infectious      = [s2.Isc, s2.I, s2.E, s2.Dx, intersect(s2.Tx, s2.mdr)];
s2['infectious_wosc'] = np.concatenate([s2['I'], s2['E'], s2['Dx'], np.intersect1d(s2['Tx'], s2['mdr'])]) #s2.infectious_wosc = [s2.I, s2.E, s2.Dx, intersect(s2.Tx, s2.mdr)];    % Infectious compartment without subclinical (contributing on TB death)
s2['prevalent']       = np.unique(np.concatenate([s2['infectious'], np.concatenate([s2['Tx'],s2['Tx2']])])) #s2.prevalent       = unique([s2.infectious, [s2.Tx,s2.Tx2]]);


#% --- Include the auxiliaries ---------------------------------------------
names = ['inc','noti','noti2','mort']
lgths =     [3,     3,      1,     2]

if not np.isin('aux',list(i)):
    i['aux'] = {}

for ii in range(len(names)): #for ii = 1:length(names)
    inds = lim + np.array(range(lgths[ii])) #inds = lim + [1:lgths(ii)];
    i['aux'][list(names)[ii]] = inds #i.aux.(names{ii}) = inds;
    lim = inds[-1]+1 #lim = inds(end);
#end
  
if not np.isin('aux',list(i0)):
    i0['aux'] = {}  
for ii in range(len(names)): #for ii = 1:length(names)
    inds0 = lim0 + np.array(range(lgths[ii])) #inds0 = lim0 + [1:lgths(ii)];
    i0['aux'][list(names)[ii]] = inds0 #i0.aux.(names{ii}) = inds0;
    lim0 = inds0[-1]+1 #lim0 = inds0(end);
#end

i['nx']  = lim
i0['nx']= lim0
i1['nx'] = lim1
i2['nx'] = lim2
#% --- Make aggregators and selectors --------------------------------------

#% Selectors for the incidence
agg = {}
tmp = np.zeros((3,i['nstates'])) #tmp = zeros(3,i.nstates);
tmp[0,s['Isc']] = 1 #tmp(1,s.Isc) = 1;                                            % All
tmp[1,[np.intersect1d(s['Isc'],s['h1']),np.intersect1d(s['Isc'],s['hart'])]] = 1#tmp(2,[intersect(s.Isc,s.h1),intersect(s.Isc,s.hart)]) = 1;  % hiv-TB
tmp[2,np.intersect1d(s['Isc'],s['mdr'])] = 1 #tmp(3,intersect(s.Isc,s.mdr)) = 1;                           % mdr-TB 
agg['inc'] = sparse.csc_matrix(tmp) #agg.inc = sparse(tmp);

sel ={}
tmp = np.zeros((i['nstates'],i['nstates'])) #tmp = zeros(i.nstates);
tmp[s['Isc'],:] = 1 #tmp(s.Isc,:) = 1;
tmp[s['h1'],s['h0']] = 0; tmp[s['hart'],s['h1']] = 0; tmp[s['h0'],s['h1']] = 0; tmp[s['h1'],s['hart']] = 0 #tmp(s.h1,s.h0) = 0; tmp(s.hart,s.h1) = 0; tmp(s.h0,s.h1) = 0; tmp(s.h1,s.hart) = 0;
tmp[s['ds'],s['mdr']]= 0; tmp[s['mdr'],s['ds']] = 0 #tmp(s.ds,s.mdr)= 0; tmp(s.mdr,s.ds) = 0;   % ADD
sel['inc'] = tmp - np.diag(np.diag(tmp)) #sel.inc = tmp - diag(diag(tmp));

tmp = np.zeros((i['nstates'],i['nstates'])) #tmp = zeros(i.nstates);
tmp[np.intersect1d(s['Tx'],s['mdr']),np.intersect1d(s['Tx'],s['ds'])] = 1 #tmp(intersect(s.Tx,s.mdr),intersect(s.Tx,s.ds)) = 1;
sel['acqu'] = sparse.csc_matrix(tmp - np.diag(np.diag(tmp))) #sel.acqu = sparse(tmp - diag(diag(tmp)));

#% Selectors for public sector notifications
tmp = np.zeros((3,i['nstates']))#tmp = zeros(3,i.nstates);
tmp[0,np.intersect1d(np.intersect1d([s['Tx'],s['Tx2']],s['h0']),s['pu'])] = 1 #tmp(1,intersect(intersect([s.Tx,s.Tx2],s.h0),s.pu)) = 1;  % public sector notification
tmp[1,np.intersect1d(np.intersect1d([s['Tx'],s['Tx2']],s['h1']),s['pu'])] = 1 #tmp(2,intersect(intersect([s.Tx,s.Tx2],s.h1),s.pu)) = 1;
tmp[2,np.intersect1d(np.intersect1d([s['Tx'],s['Tx2']],s['hart']),s['pu'])] = 1 #tmp(3,intersect(intersect([s.Tx,s.Tx2],s.hart),s.pu)) = 1;
agg['noti'] = sparse.csc_matrix(tmp) #agg.noti = sparse(tmp);

tmp = np.zeros((i['nstates'],i['nstates'])) #tmp = zeros(i.nstates);
tmp[[s['Tx'],s['Tx2']],:] = 1 #tmp([s.Tx,s.Tx2],:) = 1;
tmp[s['h1'],s['h0']] = 0; tmp[s['hart'],s['h1']] = 0; tmp[s['h0'],s['h1']] = 0; tmp[s['h1'],s['hart']] = 0 #tmp(s.h1,s.h0) = 0; tmp(s.hart,s.h1) = 0; tmp(s.h0,s.h1) = 0; tmp(s.h1,s.hart) = 0;
tmp[s['ds'],s['mdr']]= 0; tmp[s['mdr'],s['ds']] = 0
sel['noti'] = tmp - np.diag(np.diag(tmp))

#% Selectors for second-line notifications
tmp = np.zeros((i['nstates'],i['nstates']))
tmp[s['Tx2'],:] = 1
tmp[s['h1'],s['h0']] = 0; tmp[s['hart'],s['h1']] = 0; tmp[s['h0'],s['h1']] = 0; tmp[s['h1'],s['hart']] = 0
tmp[s['ds'],s['mdr']]= 0; tmp[s['mdr'],s['ds']] = 0
sel['noti2'] = tmp - np.diag(np.diag(tmp))


#% Selectors for the incidence from the full model with vaccine
agg0={}
tmp = np.zeros((3,i0['nstates']))
tmp[0,s0['Isc']] = 1  #                                          % All
tmp[1,np.concatenate([np.intersect1d(s0['Isc'],s0['h1']),np.intersect1d(s0['Isc'],s0['hart'])])] = 1 #;  % hiv-TB
tmp[2,np.intersect1d(s0['Isc'],s0['mdr'])] = 1                           #% mdr-TB 
agg0['inc'] = sparse.csc_matrix(tmp)

sel0 = {}
tmp = np.zeros((i0['nstates'],i0['nstates']))
tmp[s0['Isc'],:] = 1
tmp[s0['h1'],s0['h0']] = 0; tmp[s0['hart'],s0['h1']] = 0; tmp[s0['h0'],s0['h1']] = 0; tmp[s0['h1'],s0['hart']] = 0
tmp[s0['ds'],s0['mdr']]= 0; tmp[s0['mdr'],s0['ds']] = 0 #;   % ADD
sel0['inc'] = tmp - np.diag(np.diag(tmp))

tmp = np.zeros((i0['nstates'],i0['nstates']))
tmp[np.intersect1d(s0['Tx'],s0['mdr']),np.intersect1d(s0['Tx'],s0['ds'])] = 1
sel0['acqu'] = sparse.csc_matrix(tmp - np.diag(np.diag(tmp)))

#% Selectors for public sector notifications
tmp = np.zeros((3,i0['nstates']))
tmp[0,np.intersect1d(np.intersect1d([s0['Tx'],s0['Tx2']],s0['h0']),s0['pu'])] = 1 #tmp(1,intersect(intersect([s0.Tx,s0.Tx2],s0.h0),s0.pu)) = 1;  % public sector notification
tmp[1,np.intersect1d(np.intersect1d([s0['Tx'],s0['Tx2']],s0['h1']),s0['pu'])] = 1 #tmp(2,intersect(intersect([s0.Tx,s0.Tx2],s0.h1),s0.pu)) = 1;
tmp[2,np.intersect1d(np.intersect1d([s0['Tx'],s0['Tx2']],s0['hart']),s0['pu'])] = 1 #tmp(3,intersect(intersect([s0.Tx,s0.Tx2],s0.hart),s0.pu)) = 1;
agg0['noti'] = sparse.csc_matrix(tmp)

tmp = np.zeros((i0['nstates'],i0['nstates']))
tmp[[s0['Tx'],s0['Tx2']],:] = 1
tmp[s0['h1'],s0['h0']] = 0; tmp[s0['hart'],s0['h1']] = 0; tmp[s0['h0'],s0['h1']] = 0; tmp[s0['h1'],s0['hart']] = 0
tmp[s0['ds'],s0['mdr']]= 0; tmp[s0['mdr'],s0['ds']] = 0
sel0['noti'] = tmp - np.diag(np.diag(tmp))

#% Selectors for second-line notifications
tmp = np.zeros((i0['nstates'],i0['nstates']))
tmp[s0['Tx2'],:] = 1
tmp[s0['h1'],s0['h0']] = 0; tmp[s0['hart'],s0['h1']] = 0; tmp[s0['h0'],s0['h1']] = 0; tmp[s0['h1'],s0['hart']] = 0
tmp[s0['ds'],s0['mdr']]= 0; tmp[s0['mdr'],s0['ds']] = 0
sel0['noti2'] = tmp - np.diag(np.diag(tmp))

#% --- Define variables and ranges -----------------------------------------
#Change here
#names = ['r_beta','rfbeta_mdr','rfbeta_hiv','r_sym','p_pu','r_cs','r_cs2','r_mort_TB','p_Dx','p_Tx_complete','p_MDRrec2019', 'r_MDR_acqu','r_ART_init','r_HIV_mort','r_self_cure','p_HIV_relrate'];  # % <--- Updated
#lgths =        [1,           1,           1,     1,     1,     1,      1,          2,     2,              2,             1,            1,          1,           1,            1,            1]
#names = ['r_beta','rfbeta_mdr','rfbeta_hiv','r_sym','p_pu','r_cs','r_cs2','r_mort_TB','p_Dx','p_Tx_complete','p_MDRrec2022', 'r_MDR_acqu','r_ART_init','r_HIV_mort','r_self_cure','p_HIV_relrate', 'r_betared'];  # % <--- Updated
#lgths =        [1,           1,           1,     1,     1,     1,      1,          2,     2,              2,             1,            1,          1,           1,            1,            1, 1]

names = ['r_beta','rfbeta_mdr','rfbeta_hiv','r_sym','p_pu','r_cs','rf_cs2','r_mort_TB','p_Dx','p_Tx_complete','p_MDRrec2022', 'r_MDR_acqu','r_ART_init','r_HIV_mort','r_self_cure','p_HIV_relrate', 'rf_p_imm', 'rf_cs2022', 'r_betared'] #;   %'rf_LTBI_cure', <--- Updated
lgths =        [1,           1,           1,     1,     1,     1,      1,          2,     2,              2,             1,            1,            1,           1,            1,               1,           1,          1,         1]


xi = {}; lim = 0
for ii in range(len(names)): #for ii = 1:length(names)
    inds = lim + np.array(range(lgths[ii]))
    xi[names[ii]] = inds.astype(int) #xi.(names{ii}) = inds;
    #if names[ii]=='r_mort_TB':
    #    xxx = 0
    lim = lim + lgths[ii] #lim = inds(end);
#end
#%xi.calib = xi.r_HIV_mort;
xi['nx'] = lim

bds = np.zeros((xi['nx'],2))
prm = {}
bds[xi['r_beta'],:]       = np.array([0, 40]) #; % ds
bds[xi['rfbeta_mdr'],:]   = np.array([0, 1])
bds[xi['rfbeta_hiv'],:]   = np.array([0, 1])
bds[xi['r_sym'],:]        = np.array([0.1, 100])
bds[xi['p_pu'],:]         = np.array([0, 1])
bds[xi['r_cs'],:]         = np.array([0.1, 10]) #bds[xi['r_cs'],:]         = np.array([0.1, 100])
bds[xi['rf_cs2'],:]        = np.array([1, 40])#bds(xi.rf_cs2,:)        = [1 40]; ; #bds[xi['r_cs2'],:]        = np.array([1, 24])
#%bds(xi.r_Tx_init,:]   = [0.1 10];
bds[xi['r_mort_TB'],:]    = 1/6*np.array([[0, 2], [0, 100]]) #1/6*[0 2; 0 100];
bds[xi['p_Dx'],:]         = [np.array([0.75, 0.9]), np.array([0.1,  0.7])] #[0.75 0.9; 0.1  0.3]; % 0.4 0.7];  
bds[xi['p_Tx_complete'],:]= [np.array([0.75, 0.95]), np.array([0.4, 0.8])] #[0.75 0.95; 0.4 0.8];
bds[xi['p_MDRrec2022'],:] = np.array([0,  1])#bds[xi['p_MDRrec2019'],:] = np.array([0,  1])
bds[xi['r_MDR_acqu'],:]   = np.array([0,  0.06])
#% bds(xi.r_HIV_acqu,:)  = [0 10];
bds[xi['r_ART_init'],:]   = np.array([0, 10])
bds[xi['r_HIV_mort'],:]   = np.array([0, 10])
bds[xi['r_self_cure'],:]  = 1/6*np.array([0.85, 1.15])
bds[xi['p_HIV_relrate'],:]= np.array([1, 100])

bds[xi['rf_p_imm'],:]      = np.array([0.75, 1.25]) #bds(xi.rf_p_imm,:)      = [0.75, 1.25]; %[0.1 0.75];
bds[xi['rf_cs2022'],:]     = np.array([1, 10]) #bds(xi.rf_cs2022,:)     = [1 10];
#%bds(xi.rf_LTBI_cure,:)  = [0.75, 1.25];

bds[xi['r_betared'],:]     = np.array([0.8, 1]) #bds(xi.r_betared,:)     = [0.8 1];

prm['bounds'] = bds.T #prm.bounds = bds';
#% --- Define baseline parameter values ------------------------------------


#% Inter-age contact matrix 
#%prm.contact = [[1.3710,1.4317,0.0500];[2.5175,2.9007,0.0993];[0.2789,0.3350,0.0160]];% %Age group(0-24,25-60,60+) 
#%prm.contact = ones(3,3);% %Age group(0-24,25-60,60+) 
#% Age-group of Population in each compartment [0-4,5-10,11-15,16-65,65+]   
prm['N']   = np.array([0.2, 0.2, 0.2, 0.2, 0.2]) #prm.N    =[0.2 0.2 0.2 0.2 0.2]; %[0.39 0.31 0.24   0.021  0.032]; %1970

#% Contact matrix correcting with population weight
prm['cm'] = [[0.9591,    0.4910,    0.1820,    0.6734,    0.0088],
            [0.4908,    2.1476,    0.7891,    0.8469,    0.0191],
            [0.2415,    0.5639,    2.4042,    1.3988,    0.0219],
            [1.2417,    1.0995,    1.1480,   10.0745,    0.0586],
            [0.0289,    0.0264,    0.0266,    0.0517,    0.0063]]
#%prm.cm = prm.contact./repmat(prm.N,size(prm.N,2),1); % 5 age groups
#%prm.gth = [0 0.02 0.02 0.02]; %[0.056, 0.037, 0.034, 0.02] ; %prm.growth = 0.02;

prm['gth'] = [0, 0.039, 0.039, 0.035] #prm.gth = [0 0.039 0.039 0.035]; %[0.056, 0.037, 0.034, 0.02] ; %prm.growth = 0.02;

r={}
r['mort'] = [0.012, 0.0012, 0.0009, 0.0048, 0.0622] #r.mort = [0.012, 0.0012, 0.0009, 0.0048, 0.0622];  % Average 1997-2019
r['ageing'] = [0.186,  0.184,  0.162,  0.0047] #r.ageing = [0.186  0.184  0.162  0.0047]; % Estimated using simple model run

#% Natural history (h0, h1, hart)
r['progression']  = 0.0826*np.array([1, 10, 10*0.4])*np.array([1, 1, 0.5]) #r.progression  = 0.0826*[1 10 10*0.4].*[1 1 0.5];
r['LTBI_stabil']  = 0.872*np.array([1, 0, 1]) #r.LTBI_stabil  = 0.872*[1 0 1];
r['reactivation'] = 0.0006*np.array([1, 100, 100*0.4])*np.array([1, 1, 0.5]) #r.reactivation = 0.0006*[1 100 100*0.4].*[1 1 0.5];
#% r.self_cure    = 1/6*[1 0 1];
#% r.mort_TB      = 1/6;
r['relapse']      = np.array([0.032, 0.14, 0.0015]) #r.relapse      = [0.032 0.14 0.0015];
p = {}
p['imm']          = np.array([0.8, 0, 0.8]) #p.imm          = [0.8 0 0.8];

#% Diagnosis stage
#% r.cs           = 1;
#% p.pu           = 0.5;
#% p.Dx           = 0.65; % 0.85; 
r['Dx']             = 52 #r.Dx             = 52;
#% r.cs2          = 2;

p['MDR_rec2015'] = np.array([0.001, 0]) #p.MDR_rec2015 = [0.001 0]; % p.MDR_rec2012  = [0.036 0]; % %CHECK for each country [pu pr]
p['Tx_init2']    = np.array([0.88, 0]) #p.Tx_init2    = [0.88 0] #p.Tx_init2    = [0.88 0]; %CHECK for each country [pu pr]
p['SL_trans']    = np.array([0.88, 0]) #; %[0.2 0]; [pu pr] #p.SL_trans    = [0.88 0]; %[0.2 0]; [pu pr]

#% Treatment stage
p['Tx_init']     = np.array([1, 1])#p.Tx_init     = [1 1];
r['Tx']      = 2 #r.Tx      = 2;
r['Tx2']     = 0.5 #r.Tx2     = 0.5;   
#% p.default = 1 - 0.71;
#% r.default = r.Tx*p.default./(1-p.default);
p['cure']    = np.array([1, 1]) #p.cure    = [1 1];

p['tsrsl']     = np.array([0.48, 1e-6]) #p.tsrsl     = [0.48 1e-6];                % Second-line treatment success rate in pu and pr
r['default2']  = r['Tx2']*(1-p['tsrsl'])/p['tsrsl'] #r.default2  = r.Tx2*(1-p.tsrsl)./p.tsrsl;
p['cure2']     = np.array([0.5, 0]) #p.cure2     = [0.5 0];                 % Should I consider a different value for those with HIV?

#% Intervention parameter
#% r.pt          = 0;
p['PT_PLHIV']    = np.array([1, 1, 1]) #p.PT_PLHIV    = [1 1 1]; % With PT intervenion among PLHIv [1 1 (1-0.6)]
r['access']      = 0#r.access      = 0;
r['cs3']         = 0 #r.cs3         = 0;          % case finding rate among subclinical TB
p['VE']          = np.array([0, 0]) #p.VE          = [0 0];
r['vacc']        = np.array([0, 0, 0, 0, 0]) #r.vacc        = [0 0 0 0 0]; 
r['waning']      = 0 #r.waning      = 0;

#% Bring them all together
prm['p'] = p; prm['r'] = r #prm.p = p; prm.r = r;
ref = {}
ref['i'] = i;   ref['s'] = s;   ref['d'] = d;  ref['xi'] = xi #ref.i = i;   ref.s = s;   ref.d = d;  ref.xi = xi;
#%ref.i0 = i0; ref.s0 = s0; ref.d0 = d0; 
ref['i1'] = i1; ref['s1'] = s1; ref['d1'] = d1 #ref.i1 = i1; ref.s1 = s1; ref.d1 = d1; 
ref['i2'] = i2; ref['s2'] = s2; ref['d2'] = d2 #ref.i2 = i2; ref.s2 = s2; ref.d2 = d2; 

#% --- Get calibration targets ---------------------------------------------
#% 2019

##LOOKS LIKE DATA ARE MISSING. SHOULD BE FIXED BEFORE Setup_Model can be run
#load('../Collect_data/NGA.mat')

# #Loading NGA specific data from JSON File
# f_data_ctr = open('JSON_data.JSON')
# data_ctr = json.load(f_data_ctr)
# data = data_ctr['data']
# prm = data_ctr['prm']
# r = data_ctr['r']
# r = data_ctr['p']
# xs = data_ctr['xs']

# #% Age-group of Population in each compartment [0-4,5-9,10-14,15-65,65+]   
# prm['N'] = np.array([0.2, 0.2, 0.2, 0.2, 0.2]) #prm.N = [0.2 0.2 0.2 0.2 0.2]; % Initial population (assumption)
# prm['cm'] = data['contact_mat'] #prm.cm = data.contact_mat;     % Contact matrix correcting with population weight
# prm['gth'] = np.array([0, 0.03, 0.035, 0.044]) #prm.gth = [0 0.03 0.035 0.044]; %[0,data.gth];        % population growth rate
# r['mort'] = data['age_mort'] #r.mort = data.age_mort';        % Age specific mortality rate
# r['ageing'] = np.array([0.1774, 0.1696, 0.1594, 0.0050]) #r.ageing = [0.1774 0.1696 0.1594 0.0050]; %data.ageing';   % Ageing rate

# #% --- Get calibration targets ---------------------------------------------
# #%data.pop00_22 = data.pop00_22;
# popn          = data['pop00_22'][-1] #popn          = data.pop00_22(end); % 2022
# data['inc2015']  = np.array(data['inc_all'])[np.array(data['inc_all']).shape[0]-8,:] #data.inc2015  = data.inc_all(end-7,:); 
# data['inc2022']  = np.array(data['inc_all'])[np.array(data['inc_all']).shape[0]-1,:] #data.inc2022  = data.inc_all(end,:); 
# #% data.inc_h1  = data.inc_h1;
# #% data.mdr2015  = data.mdr2015; 
# #% data.mdr2022  = data.mdr2022;
# data['mort_H0']  = np.array(data['mort_H0'])[np.array(data['mort_H0']).shape[0]-1,:] #data.mort_H0  = data.mort_H0(end,:);
# #%data.mort_H1  = data.mort_H1;
# data['noti']     = np.array([0.8, 1, 1.2])*np.array(data['noti']) #data.noti     = [0.8 1 1.2]*data.noti;
# data['mdriniTX'] = np.array([0.8, 1, 1.2])*np.array(data['mdriniTX']) #data.mdriniTX = [0.8 1 1.2]*data.mdriniTX; 
# data['sym']      = np.array([0.36, 0.50, 0.80])#data.sym      = [0.36 0.50 0.80];        % proportion of prevalent TB that has symptoms

# #% Get the HIV data                                                         
# prm['ART_start'] = data['ART_start'] #prm.ART_start = data.ART_start;                     %ART_start;
# data['ART_covg'] =  np.array([0.85, 1, 1.15])*data['ART_covg'] #data.ART_covg =  [0.85 1 1.15]*data.ART_covg;       %ARTcovg_2022;
# data['HIV_prev'] = np.array([0.85, 1, 1.15])*(data['HIV_prev']/popn) #data.HIV_prev = [0.85 1 1.15]*(data.HIV_prev./popn);%HIVprev_2022 (proportion of population)
# HIV_incd = data['HIV_incd'] #HIV_incd = data.HIV_incd;                           %HIV incidence (proportion of population) 1980-2022


# #% Extrapolate HIV incidence by 20 years
# ys1 = HIV_incd #;   %1980-2022
# xs1 = np.array(range(len(ys1))) #xs1 = 1:length(ys1);
# xs2 = np.array(range(len(ys1)))+20#xs2 = 1:length(ys1)+20;    % 2022-2042
# #ys2 = interp1(xs1,ys1,xs2,'linear','extrap');  % linier trend for the next 21 years
# fnys2 = interpolate.interp1d(xs1,ys1,kind='linear',fill_value='extrapolate')
# ys2 = [fnys2(o) for o in xs2]

# prm['rHIV'] = ys2 #prm.rHIV = ys2;


# #% num = [data.inc_all; data.inc_h1; data.noti; data.ART_covg; data.HIV_prev; data.mort_H0; data.mort_H1;data.mdr2015;data.mdr2019;data.mdriniTX];
# show = 0
# f0 = get_distribution_fns(data['inc2015'],   'lognorm', show) #f0 = get_distribution_fns(data.inc2015,   'lognorm', show)
# f1 = get_distribution_fns(data['inc2022'],   'lognorm', show) #f1 = get_distribution_fns(data['inc_all'],   'lognorm', show), changed
# f2 = get_distribution_fns(data['inc_h1'],    'lognorm', show)
# f3 = get_distribution_fns(data['noti'],      'lognorm', show)
# f4 = get_distribution_fns(data['ART_covg'],  'lognorm', show)
# f5 = get_distribution_fns(data['HIV_prev'],  'lognorm', show)
# f6 = get_distribution_fns(data['mort_H0'],   'lognorm', show)
# f7 = get_distribution_fns(data['mort_H1'],   'lognorm', show)
# f8 = get_distribution_fns(data['mdr2015'],   'lognorm', show)
# f9 = get_distribution_fns(data['mdr2022'],   'lognorm', show) #f9 = get_distribution_fns(data['mdr2019'],   'lognorm', show), changed
# f10 = get_distribution_fns(data['mdriniTX'], 'lognorm', show); #%show ->0
# f11 = get_distribution_fns(data['sym'],      'lognorm', show)
      
# def fn(inc2015, inc2022, inc_h1, noti, ART_covg, HIV_prev, mort_H0, mort_H1, mdr2015, mdr2019, mdriniTX,sym):
#         res_0 = 10*f1[0](inc2015)
#         res_1 = 10*f1[0](inc2022)
#         res_2 = 10*f2[0](inc_h1) 
#         res_3 = f3[0](noti) 
#         res_4 = f4[0](ART_covg)
#         res_5 = f5[0](HIV_prev) 
#         res_6 = f6[0](mort_H0) 
#         res_7 = f7[0](mort_H1)
#         res_9 = 10*f9[0](mdr2019) 
#         res_10 = f10[0](mdriniTX)
#         res_11 = f11[0](sym) #; %+ f8(mdr2015)
#         result = res_0 + res_1 + res_2 + res_3 + res_4 + res_5 + res_6 + res_7 + res_9 + res_10 + res_11
#         return result


# lhd = {}
# lhd['fn']  = fn  
# lhd['sgn'] = -math.inf

#save Model_setup;
#* Get all local variables as a dictionary
variables_dict = locals()

#* Filter out unwanted variables (like built-in ones)
variables_dict = {k: v for k, v in variables_dict.items() if not (k.startswith('__') or  k in {'gc', 'gc', 'get_addresses',
'np', 'scipy', 'sparse', 'interpolate', 'get_distribution_fns', 'math', 'pickle','json', 'show', 'variables_dict', 'f_data_ctr', 'f0',
'f1', 'f2', 'f3', 'f4', 'f5', 'f6', 'f7', 'f8', 'f9', 'f10', 'f11','fn','lhd'})}


#* Save all variables to a file named 'Model_setup.pkl'
with open('data/SetupModel/Model_setup.pkl', 'wb') as file:
    pickle.dump(variables_dict, file)






# Fields used in IC ModVar data structures

# general fields for keeping ID fields consistent when relating records to each other and for use in result records

IC_F_INTERV_ID = "intervID"
IC_F_DELIV_CHAN_ID = "delivChanID"
IC_F_TREAT_INPUT_ID = "treatInputID"
IC_F_TREAT_INPUT_GROUP_ID = "treatInputGroupID"
IC_F_STATIC_STAFF_MST_ID = "defaultStaffTypeMstID"  # was defaultStaffMstID
IC_F_DYNAMIC_STAFF_MST_ID = "customStaffID"
IC_F_DRUG_SUPPLY_ID = "drugSupplyID"
IC_F_DRUG_SUPPLY_TYPE_MST_ID = "drugSupplyTypeMstID"
IC_F_YEAR = "year"
IC_F_VALUE = "value"
IC_F_APPLIC_PROG_AREA_IDS = "applicProgAreaIDs"
IC_F_CUSTOM_STAFF_TYPE = "customStaffType"

# target population records (contained within intervention list)

IC_TARG_POP_NAME = "name"  # string
IC_TARG_POP_MST_ID = "targPopMstID"  # integer
IC_TARG_POP_SEX = "sex"  # integer; 1 = male, 2 = female
IC_TARG_POP_START_AGE = "startAge"  # integer; 0-80
IC_TARG_POP_END_AGE = "endAge"  # integer; 0-80
IC_TARG_POP_DIRECT_ENTRY_VALUES = "directEntryValues"  # list of floats; by year

# target population default records (contain info about each target population)

IC_TARG_POP_DEF_NAME = "name"
IC_TARG_POP_DEF_STR_CONST = "stringConstant"
IC_TARG_POP_DEF_MST_ID = "defTargPopMstID"
IC_TARG_POP_DEF_APPLIC_PROG_AREA_IDS = IC_F_APPLIC_PROG_AREA_IDS
IC_TARG_POP_DEF_REQ_MOD_MST_IDS = "requiredModMstIDs"

# intervention records

IC_INTERV_ID = (
    IC_F_INTERV_ID  # string; numeric mstID for defaults (still cast as string). GUID for custom interventions.
)
IC_INTERV_STR_CONST = "stringConstant"  # string;  Defaults only. Ex: GB_stTetanusToxoid.
IC_INTERV_NAME = "name"  # string;  Holds custom name or overridden default name.
IC_INTERV_ACTIVE = "active"  # boolean; Is user using this item or not?
IC_INTERV_PROG_AREA_ID = "progAreaID"  # string;  ID from programme area records.
IC_INTERV_SUBGROUP_ID = "subgroupID"  # string;  ID from subgroup records.
IC_INTERV_IMPACT = "impact"  # boolean; yes = gets coverage from Impact module like LiST. No otherwise.
IC_INTERV_CUSTOM = "custom"  # boolean; user-entered = true; default = false.
IC_INTERV_SPECIAL_CASES = (
    "specialCases"  # list;    Flags (strings) that tell us if an intervention should have a special case applied to it
)
IC_INTERV_STRATIFYING = "stratifying"  # boolean;
IC_INTERV_COST_FOR_PHC = "costForPHC"  # boolean;
IC_INTERV_PIN_EDITOR_VISITED = "PIN_EditorVisited"  # boolean;
IC_INTERV_COVERAGE_CHANGED = "coverageChanged"  # boolean;
IC_INTERV_PIN_STR_CONSTANT = "PIN_StringConstant"  # string; from Intervention DB // not saved out to file on desktop
IC_INTERV_COVERAGE = "coverage"  # list of float; by year
IC_INTERV_TREAT_INPUT_NOT_REQ = "treatInputNotReq"  # 2d list of boolean; delivery channel x treatment input type
IC_INTERV_MAX_CUSTOM_TI_GROUP_MST_ID = (
    "maxCustomTIGroupMstID"  # 2d list of integer; delivery channel x treatment input type // 'purpose group'
)
# Moved these out because they were huge
# IC_INTERV_DRUG_COST_PER_CASE             = 'drugCostPerCase'             # 2d list of float; delivery channel x year
# IC_INTERV_SUPPLY_COST_PER_CASE           = 'supplyCostPerCase'           # 2d list of float; delivery channel x year
# IC_INTERV_MIN_PER_AVG_CASE_DEFAULT_STAFF = 'minPerAvgCaseDefaultStaff'   # 3d list of float; delivery channel x default staff type x year
# IC_INTERV_MIN_PER_AVG_CASE_CUSTOM_STAFF  = 'minPerAvgCaseCustomStaff'    # 3d list of float; delivery channel x custom staff type x year
# IC_INTERV_LABOR_COST_PER_CASE            = 'laborCostPerCase'            # 2d list of float; delivery channel x year
IC_INTERV_PIN = "PIN"  # list of float; by year
IC_INTERV_PERC_DELIV_THROUGH_CHAN = "percDelivThroughChan"  # 2d list of float; yearID x IC deliv channel
IC_INTERV_VISIT_COST_PER_CASE = "visitCostPerCase"  # 3d list of float; visit type x delivery channel x year
IC_INTERV_TARG_POP_REC = "targPopRec"  # targ pop dict

# Alternative coverage records

IC_ALT_COV_INTERV_ID = "intervID"
IC_ALT_COV_VALUE = "altCoverage"

# Alternative PIN age group set 1 records

IC_ALT_PIN_AGE_GROUP_SET_1_INTERV_ID = "intervID"
IC_ALT_PIN_AGE_GROUP_SET_1_VALUE = "PIN"

# Alternative PIN age group set 2 records

IC_ALT_PIN_AGE_GROUP_SET_2_INTERV_ID = "intervID"
IC_ALT_PIN_AGE_GROUP_SET_2_VALUE = "PIN"

# drug and supply records (NOT the same as treatment inputs but used to create drug and supply treatment inputs)

IC_DRUG_SUPPLY_ID = IC_F_DRUG_SUPPLY_ID  # string; numeric mstID for defaults (still cast as string). GUID for custom drugs and supplies.
IC_DRUG_SUPPLY_NAME = "name"  # string;  Holds custom name or overridden default name.
IC_DRUG_SUPPLY_STR_CONST = "stringConstant"  # string;  Defaults only.
IC_DRUG_SUPPLY_COST_TYPE_LG_MST_ID = "costTypeMstID"  # integer;
IC_DRUG_SUPPLY_ACTIVE = "active"  # boolean; Is user using this item or not?
IC_DRUG_SUPPLY_CUSTOM = "custom"  # boolean; user-entered = true; default = false.
IC_DRUG_SUPPLY_TYPE_MST_ID = IC_F_DRUG_SUPPLY_TYPE_MST_ID  # integer; 1 = drug, 2 = supply
IC_DRUG_SUPPLY_APPLIC_PROG_AREA_LIST = (
    IC_F_APPLIC_PROG_AREA_IDS  # list of programme area mstIDs for which the drug/supply should appear
)
IC_DRUG_SUPPLY_UNIT_COST = "unitCost"  # list of floats; by year
IC_DRUG_SUPPLY_LOG_SUPP_CHAIN_COSTS_PERC = "logisticsSupplyChainCostsPerc"  # int; TB app only
IC_DRUG_SUPPLY_WASTAGE_PERC = "wastagePerc"  # int; TB app only
# For Logistics
IC_DRUG_SUPPLY_PERC_WASTAGE = "percWastage"  # integer
IC_DRUG_SUPPLY_PERC_SAFETY_STOCK_NEEDED = "percSafetyStockNeeded"  # integer
IC_DRUG_SUPPLY_SAFETY_STOCK_BASELINE = "safetyStockBaseline"  # integer
IC_DRUG_SUPPLY_PERC_SAFETY_STOCK_REC = "percSafetyStockRecommended"  # 1d list of integer; by year
IC_DRUG_SUPPLY_QUANTITY = "quantity"  # 1d list of integer; by year
IC_DRUG_SUPPLY_UNIT_WEIGHT = "unitWeight"  # float
IC_DRUG_SUPPLY_UNIT_VOLUME = "unitVolume"  # float
IC_DRUG_SUPPLY_REQUIRES_COLD_STORAGE = "requiresColdStorage"

# treatment input group (TIG) records

IC_TIG_ID = (
    IC_F_TREAT_INPUT_GROUP_ID  # string; numeric mstID for defaults (still cast as string). GUID for custom groups
)
IC_TIG_INTERV_ID = IC_F_INTERV_ID  # string; numeric mstID for defaults (still cast as string). GUID for custom groups
IC_TIG_DELIV_CHAN_ID = (
    IC_F_DELIV_CHAN_ID  # string; numeric mstID for defaults (still cast as string). GUID for custom groups and supplies
)
IC_TIG_NAME = "name"  # string
IC_TIG_STR_CONST = "stringConstant"  # string;  Defaults only.

# treatment input (TI) record common fields

IC_TI_ID = (
    IC_F_TREAT_INPUT_ID  # string; (either a drug/supply ID, personnel ID, or outpatient visit/inpatient visit ID)
)
IC_TI_INTERV_ID = IC_F_INTERV_ID  # string; numeric mstID for defaults (still cast as string). GUID for custom groups
IC_TI_DELIV_CHAN_ID = (
    IC_F_DELIV_CHAN_ID  # string; numeric mstID for defaults (still cast as string). GUID for custom delivery channels
)
IC_TI_GROUP_ID = (
    IC_F_TREAT_INPUT_GROUP_ID  # string; numeric mstID for defaults (still cast as string). GUID for custom groups
)
IC_TI_PERC_APPLIED = "percApplied"  # float
IC_TI_NOTE = "note"  # string
IC_TI_NOTE_STR_CONST = "stringConstant"  # string;  Defaults only.

# treatment input drug and supply record fields

IC_TI_DRUG_SUPPLY_TIMES_PER_DAY = "timesPerDay"  # integer
IC_TI_DRUG_SUPPLY_DAYS_PER_CASE = "daysPerCase"  # integer
IC_TI_DRUG_SUPPLY_NUMBER = "number"  # integer

# treatment input medical personnel record fields

IC_TI_MED_PERSONNEL_CUSTOM_STAFF_TYPE = IC_F_CUSTOM_STAFF_TYPE  # boolean
IC_TI_MED_PERSONNEL_SKILLED_STAFF_COST = "skilledStaffCost"  # boolean; GCEA
IC_TI_MED_PERSONNEL_NUM_OF_DAYS = "numOfDays"  # integer
IC_TI_MED_PERSONNEL_MINUTES = "minutes"  # float

# treatment input outpatient visits and inpatient day record fields

IC_TI_OUTPAT_INPAT_UNITS_PER_CASE = "unitsPerCase"  # double



# fmt: off
#######################################################################################################
#                                                                                                     #
#                                       Drug Supply DB (DSDB)                                         #
#                                                                                                     #
#######################################################################################################

IC_MST_ID_KEY_DSDB                       = 'mstID'
IC_CONSTANT_KEY_DSDB                     = 'constant'
IC_ENGLISH_STRING_KEY_DSDB               = 'englishString'
IC_STR_CONST_KEY_DSDB                    = 'stringConstant' 
IC_TYPE_MST_ID_KEY_DSCB                  = 'typeMstID'
IC_UNIT_COST_KEY_DSCB                    = 'unitCost'
IC_FAMILY_KEY_LG_DSCB                    = 'family'
IC_WEIGHT_KEY_LG_DSCB                    = 'weight'
IC_UOM_KEY_LG_DSCB                       = 'weightUOM'
IC_VOLUME_KEY_LG_DSCB                    = 'volume'
IC_VOLUME_UOM_KEY_LG_DSCB                = 'volumeUOM'
IC_FORECAST_ERROR_KEY_LG_DSCB            = 'forecastError'
IC_IMPORTANCE_KEY_LG_DSCB                = 'importance'
IC_NON_CRIT_FRACTION_KEY_LG_DSCB         = 'nonCriticalFraction'
IC_STD_FRACTION_KEY_LG_DSCB              = 'standardFraction'
IC_CRIT_FRACTION_KEY_LG_DSCB             = 'criticalFraction'
IC_COST_PER_PACK_KEY_LG_DSCB             = 'costPerPack'
IC_UNITS_PER_PACK_KEY_LG_DSCB            = 'unitsPerPack'
IC_COST_TYPE_MST_ID_KEY_LG_DSCB          = 'costTypeMstID'
IC_FAMILY_KEY_LG_DSCB                    = 'family'
IC_SOURCE_KEY_DSCB                       = 'source'
IC_PROG_AREA_FILTER_MST_IDS_KEY_DSCB     = 'progAreaFilterMstIDs'
IC_COLD_STORAGE_KEY_LG_DSDB              = 'requiresColdStorage'

#######################################################################################################
#                                                                                                     #
#                                       Treatment Input DB (TIDB)                                     #
#                                                                                                     #
#######################################################################################################

IC_INTERV_MST_ID_KEY_TIDB           = 'intervMstID'
IC_DELIV_CHAN_MST_ID_KEY_TIDB       = 'delivChanMstID'
IC_TREAT_INPUT_TYPE_MST_ID_KEY_TIDB = 'treatInputTypeMstID'
IC_GROUP_MST_ID_KEY_TIDB            = 'groupMstID'
IC_TREATMENT_INPUT_MST_ID_KEY_TIDB  = 'treatmentInputMstID'
IC_PERC_APPLIED_KEY_TIDB            = 'percApplied'
IC_NOTE_KEY_TIDB                    = 'note'
IC_NOTE_STR_CONST_KEY_TIDB          = 'noteStringConstant'
IC_NUM_ITEMS_KEY_TIDB               = 'numItems'
IC_TIMES_PER_DAY_MIN_KEY_TIDB       = 'timesPerDayOrMin'
IC_DAYS_PER_CASE_NUM_DAYS_KEY_TIDB  = 'daysPerCaseOrNumDays'
IC_UNITS_PER_CASE_PERSON_KEY_TIDB   = 'unitsPerCaseOrPerson'

# Normally we don't need these here, but we do if we want to access them outside of the Tools folder.

IC_INTERV_MST_ID_TAG_TIDB             = '<Intervention Master ID>'  
IC_INTERV_NAME_TAG_TIDB               = '<Intervention Name>'
IC_SPECIAL_CASE_TYPE_MST_IDS_TAG_TIDB = '<Special Case Type Master IDs>' # Currently just for LiST Costing, so ignoring
IC_DELIV_CHAN_MST_IDS_TAG_TIDB        = '<Delivery Channel Master IDs>'
IC_TREAT_INPUT_TYPE_MST_ID_TAG_TIDB   = '<Treatment Input Type Master ID>'
IC_GROUP_MST_ID_TAG_TIDB              = '<Group Master ID>'
IC_GROUP_NAME_TAG_TIDB                = '<Group Name>'
IC_GROUP_STR_CONST_TAG_TIDB           = '<Group String Constant>'
IC_INPUT_MST_ID_TAG_TIDB              = '<Input Master ID>'
IC_INPUT_NAME_TAG_TIDB                = '<Input Name>'
IC_PERC_REC_TREATED_BY_TAG_TIDB       = '<Percent Receiving or Percent Treated By>'
IC_NOTE_TAG_TIDB                      = '<Note>'
IC_NOTE_STR_CONST_TAG_TIDB            = '<Note String Constant>'
IC_NUM_ITEMS_TAG_TIDB                 = '<Number of Items>'
IC_TIMES_PER_DAY_MIN_TAG_TIDB         = '<Times per Day or Minutes>'
IC_DAYS_PER_CASE_NUM_DAYS_TAG_TIDB    = '<Days per Case or Number of Days>'
IC_UNITS_PER_CASE_PERSON_TAG_TIDB     = '<Units per Case or Units per Person>'

#######################################################################################################
#                                                                                                     #
#                                       Treatment Input GroupDB (TIGDB)                               #
#                                                                                                     #
#######################################################################################################

IC_INTERV_MST_ID_KEY_TIGDB           = 'intervMstID'
IC_DELIV_CHAN_MST_ID_KEY_TIGDB       = 'delivChanMstID'
IC_TREAT_INPUT_TYPE_MST_ID_KEY_TIGDB = 'treatInputTypeMstID'
IC_GROUP_MST_ID_KEY_TIGDB            = 'groupMstID'
IC_GROUP_NAME_KEY_TIGDB              = 'groupName'
IC_GROUP_STR_CONST_KEY_TIGDB         = 'groupStringConstant'

#######################################################################################################
#                                                                                                     #
#                                       PIN DB (PINDB)                                                #
#                                                                                                     #
#######################################################################################################

IC_ISO_ALPHA_3_COUNTRY_CODE_KEY_PINDB = 'ISO3AlphaCountryCode'
IC_INTERVENTION_LIST_KEY_PINDB        = 'interventions'

IC_INTERV_MST_ID_KEY_PINDB = 'intervMstID'
IC_PIN_KEY_PINDB           = 'PIN'

#######################################################################################################
#                                                                                                     #
#                                       PIN by Year DB (PbYDB)                                        #
#                                                                                                     #
#######################################################################################################

IC_ISO_ALPHA_3_COUNTRY_CODE_KEY_PbYDB = 'ISO3AlphaCountryCode'
IC_INTERVENTION_LIST_KEY_PbYDB        = 'interventions'

IC_INTERV_MST_ID_KEY_PbYDB = 'intervMstID'
IC_PIN_KEY_PbYDB           = 'PIN'
IC_FIRST_YEAR_KEY_PbYDB    = 'firstYear'
IC_FINAL_YEAR_KEY_PbYDB    = 'finalYear'

#######################################################################################################
#                                                                                                     #
#                                       Non Impact Coverage DB (NICDB)                                #
#                                                                                                     #
#######################################################################################################

IC_ISO_ALPHA_3_COUNTRY_CODE_KEY_NICDB = 'ISO3AlphaCountryCode'
IC_INTERVENTION_LIST_KEY_NICDB        = 'interventions'

IC_INTERV_MST_ID_KEY_NICDB  = 'intervMstID'
IC_NON_IMPACT_COV_KEY_NICDB = 'nonImpactCoverage'

#######################################################################################################
#                                                                                                     #
#                                       Non Impact Coverage DB (NbYDB)                                #
#                                                                                                     #
#######################################################################################################

IC_ISO_ALPHA_3_COUNTRY_CODE_KEY_NbYDB = 'ISO3AlphaCountryCode'
IC_INTERVENTION_LIST_KEY_NbYDB        = 'interventions'

IC_INTERV_MST_ID_KEY_NbYDB  = 'intervMstID'
IC_NON_IMPACT_COV_KEY_NbYDB = 'nonImpactCoverage'
IC_FIRST_YEAR_KEY_NbYDB     = 'firstYear'
IC_FINAL_YEAR_KEY_NbYDB     = 'finalYear'

#######################################################################################################
#                                                                                                     #
#                                       Target Population Direct Entry DB (TPDEDB)                    #
#                                                                                                     #
#######################################################################################################

IC_ISO_ALPHA_3_COUNTRY_CODE_KEY_TPDEDB = 'ISO3AlphaCountryCode'
IC_INTERVENTION_LIST_KEY_TPDEDB        = 'interventions'

IC_INTERV_MST_ID_KEY_TPDEDB = 'intervMstID'
IC_TARG_POP_KEY_TPDEDB      = 'targPop'

#######################################################################################################
#                                                                                                     #
#                                       Target Population DB (TPDB)                                   #
#                                                                                                     #
#######################################################################################################

IC_CONSTANT_KEY_TPDB                  = 'constant'
IC_MST_ID_KEY_TPDB                    = 'mstID'
IC_NAME_KEY_TPDB                      = 'name'
IC_STR_CONST_KEY_TPDB                 = 'stringConstant'
IC_APPLIC_PROG_AREA_IDS_IHT_KEY_TPDB  = 'applicProgAreaIDsIHT'
IC_APPLIC_PROG_AREA_IDS_TB_KEY_TPDB   = 'applicProgAreaIDsTB'
IC_APPLIC_PROG_AREA_IDS_LIST_KEY_TPDB = 'applicProgAreaIDsLiST'
IC_REQ_MOD_MST_IDS_KEY_TPDB           = 'reqModMstIDs'

#######################################################################################################
#                                                                                                     #
#                                       Adolescent Health PIN DB (AHPINDB)                            #
#                                                                                                     #
#######################################################################################################

#######################################################################################################
#                                                                                                     #
#                                       Adolescent Health Coverage DB (AHCDB)                         #
#                                                                                                     #
#######################################################################################################

#######################################################################################################
#                                                                                                     #
#                                       OutpatVisitInpatDayCostsDB (OVIDCTBDB)                          #
#                                                                                                     #
#######################################################################################################

IC_ISO_ALPHA_3_COUNTRY_CODE_KEY_OVIDCTBDB        = 'ISO3AlphaCountryCode'

IC_COST_PER_OUTPAT_VISIT_INPAT_DAY_KEY_OVIDCTBDB = 'costPerOutpatVisitInpatDay'

#######################################################################################################
#                                                                                                     #
#                                       OutpatVisitInpatDayCostsCSDB (OVIDCCSDB)                      #
#                                                                                                     #
#######################################################################################################

IC_ISO_ALPHA_3_COUNTRY_CODE_KEY_OVIDCCSDB  = 'ISO3AlphaCountryCode'

IC_OTHER_RECURRENT_COSTS_KEY_OVIDCCSDB     = 'otherRecurrentCosts'
IC_CAPITAL_COSTS_KEY_OVIDCCSDB             = 'capitalCosts'

#######################################################################################################
#                                                                                                     #
#                                       HealthSystemCostsDB (HSCDB)                                   #
#                                                                                                     #
#######################################################################################################

IC_ISO_ALPHA_3_COUNTRY_CODE_KEY_HSCDB              = 'ISO3AlphaCountryCode'

IC_SUPPLY_CHAIN_COST_PERC_COMMODITY_COST_KEY_HSCDB = 'supplyChainCostPercCommodityCost'
IC_INFRASTRUCTURE_INVESTMENT_RATIO_KEY_HSCDB       = 'infrastructureInvestmentRatio' 
IC_OTHER_TO_INVESTMENT_COSTS_RATIO_KEY_HSCDB       = 'otherToInvestmentCostsRatio'
IC_INCOME_LEVEL_KEY_HSCDB                          = 'incomeLevel'
IC_SUPPLY_CHAIN_STATUS_KEY_HSCDB                   = 'supplyChainStatus'
